@@ -2,9 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
-* Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2012 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -37,54 +35,49 @@
 #include "ui_webappcreation.h"
 
 // Local Includes
+#include "searchengine.h"
+
+#include "tabbar.h"
+#include "tabwindow.h"
+
+#include "webwindow.h"
+#include "webtab.h"
+#include "webpage.h"
+
+#include "urlresolver.h"
+
+// Local Manager(s) Includes
 #include "adblockmanager.h"
-#include "bookmarkmanager.h"
 #include "downloadmanager.h"
 #include "historymanager.h"
 #include "iconmanager.h"
-#include "mainview.h"
-#include "mainwindow.h"
-#include "opensearchmanager.h"
-#include "searchengine.h"
 #include "sessionmanager.h"
-#include "syncmanager.h"
-#include "stackedurlbar.h"
-#include "tabbar.h"
-#include "urlbar.h"
-#include "urlresolver.h"
-#include "useragentmanager.h"
-#include "webtab.h"
+
+// Ui Includes
+#include "ui_cleardata.h"
 
 // KDE Includes
 #include <KCmdLineArgs>
-#include <KIcon>
-#include <KMessageBox>
+
+#include <KDialog>
+#include <KProcess>
+#include <KPushButton>
 #include <KStandardDirs>
-#include <KStartupInfo>
-#include <ThreadWeaver/Weaver>
-#include <KAction>
 #include <KWindowSystem>
 #include <KWindowInfo>
-#include <KGlobal>
-#include <KCharsets>
-#include <KPushButton>
-#include <KMimeType>
+#include <KStartupInfo>
+
+#include <KMessageBox>
 
 // Qt Includes
-#include <QVBoxLayout>
-#include <QTimer>
-
-
-using namespace ThreadWeaver;
+#include <QDBusInterface>
+#include <QDBusReply>
+#include <QDir>
 
 
 Application::Application()
     : KUniqueApplication()
-    , _privateBrowsingAction(0)
 {
-    _privateBrowsingAction = new KAction(KIcon("view-media-artist"), i18n("Private &Browsing"), this);
-    _privateBrowsingAction->setCheckable(true);
-    connect(_privateBrowsingAction, SIGNAL(triggered(bool)), this, SLOT(setPrivateBrowsingMode(bool)));
 }
 
 
@@ -95,80 +88,7 @@ Application::~Application()
     ReKonfig::setRecoverOnCrash(0);
     saveConfiguration();
 
-    Q_FOREACH(QWeakPointer<MainWindow> window, m_mainWindows)
-    {
-        kDebug() << "deleting windows...";
-        delete window.data();
-        window.clear();
-    }
-
-    if (!m_historyManager.isNull())
-    {
-        kDebug() << "deleting history manager";
-        delete m_historyManager.data();
-        m_historyManager.clear();
-    }
-
-    if (!m_bookmarkManager.isNull())
-    {
-        kDebug() << "deleting bookmark Manager";
-        delete m_bookmarkManager.data();
-        m_bookmarkManager.clear();
-    }
-
-    if (!m_sessionManager.isNull())
-    {
-        kDebug() << "deleting session manager";
-        delete m_sessionManager.data();
-        m_sessionManager.clear();
-    }
-
-    if (!m_opensearchManager.isNull())
-    {
-        kDebug() << "deleting opensearch manager";
-        delete m_opensearchManager.data();
-        m_opensearchManager.clear();
-    }
-
-    if (!m_iconManager.isNull())
-    {
-        kDebug() << "deleting icon manager";
-        delete m_iconManager.data();
-        m_iconManager.clear();
-    }
-
-    if (!m_adblockManager.isNull())
-    {
-        kDebug() << "deleting adblock manager";
-        delete m_adblockManager.data();
-        m_adblockManager.clear();
-    }
-
-    if (!m_userAgentManager.isNull())
-    {
-        kDebug() << "deleting user agent manager";
-        delete m_userAgentManager.data();
-        m_userAgentManager.clear();
-    }
-
-    if (!m_syncManager.isNull())
-    {
-        kDebug() << "deleting sync manager";
-        delete m_syncManager.data();
-        m_syncManager.clear();
-    }
-
-    // TODO:
-    // add a check to NOT close rekonq
-    // until last download is finished
-    if (!m_downloadManager.isNull())
-    {
-        kDebug() << "deleting download manager";
-        delete m_downloadManager.data();
-        m_downloadManager.clear();
-    }
-
-    kDebug() << "Bye bye...";
+    kDebug() << "Bye bye (k)baby...";
 }
 
 
@@ -186,14 +106,61 @@ int Application::newInstance()
     bool areThereArguments = (args->count() > 0);
     bool hasToBeRecoveredFromCrash = (ReKonfig::recoverOnCrash() > 0);
     // note that hasToBeRecoveredFromCrash is always true if it is not the first load
-    // !isFirstLoad -> hasToBeRecovered
+    // !isFirstLoad -> hasToBeRecoveredFromCrash
 
     kDebug() << "is first load? " << isFirstLoad;
     kDebug() << "are there arguments? " << areThereArguments;
     kDebug() << "is rekonq crashed? " << hasToBeRecoveredFromCrash;
 
+    bool incognito = args->isSet("incognito");
+    bool webapp = args->isSet("webapp");
+
+    if (webapp)
+    {
+        kDebug() << "WEBAPP MODE...";
+        if (args->count() == 0)
+        {
+            KMessageBox::error(0, i18n("Error"), i18n("Cannot launch webapp mode without an URL to load"));
+            return 1;
+        }
+
+        if (args->count() > 1)
+        {
+            KMessageBox::error(0, i18n("Error"), i18n("Cannot load more than one URL in webapp mode"));
+            return 1;
+        }
+
+        KUrl u = args->url(0);
+        if (!u.isLocalFile() || !QFile::exists(u.toLocalFile()))
+        {
+            u = UrlResolver::urlFromTextTyped(args->arg(0));
+        }
+        kDebug() << "URL: " << u;
+
+        WebTab *tab = newWebApp();
+        connect(tab->page(), SIGNAL(pageCreated(WebPage*)), this, SLOT(pageCreated(WebPage*)));
+        tab->view()->load(u);
+
+        if (isFirstLoad)
+        {
+            // updating rekonq configuration
+            updateConfiguration();
+
+            setWindowIcon(KIcon("rekonq"));
+
+            // just create History Manager...
+            HistoryManager::self();
+        }
+
+        KStartupInfo::appStarted();
+        isFirstLoad = false;
+        return 0;
+    }
+
     if (areThereArguments)
     {
+        kDebug() << "DEFAULT MODE, WITH ARGUMENTS...";
+
         // prepare URLS to load
         KUrl::List urlList;
         for (int i = 0; i < args->count(); ++i)
@@ -201,38 +168,48 @@ int Application::newInstance()
             const KUrl u = args->url(i);
 
             if (u.isLocalFile() && QFile::exists(u.toLocalFile())) // "rekonq somefile.html" case
+            {
                 urlList += u;
+            }
             else
             {
-                // "rekonq kde.org" || "rekonq kde:kdialog" case
-                UrlResolver res(args->arg(i));
-                UrlSearchList list = res.orderedSearchItems();
-                if (list.isEmpty())
-                {
-                    urlList += u;
-                }
-                else
-                {
-                    urlList += list.first().url;
-                }
+                // "rekonq kde.org" || "rekonq kde:kdialog" cases
+                urlList += UrlResolver::urlFromTextTyped(args->arg(i));
             }
         }
 
-        if (isFirstLoad && (ReKonfig::startupBehaviour() == 2) && sessionManager()->restoreSessionFromScratch())
+        if (isFirstLoad)
         {
-            isFirstLoad = false;
+            bool restoreOk = false;
+            if (ReKonfig::startupBehaviour() == 2)
+            {
+                restoreOk = SessionManager::self()->restoreSessionFromScratch();
+            }
+            else
+            {
+                restoreOk = SessionManager::self()->restoreJustThePinnedTabs();
+            }
+
+            isFirstLoad = !restoreOk;
         }
 
         // first argument: 99% of the time we have just that...
-        if (isFirstLoad)
+        if (isFirstLoad || m_tabWindows.count() == 0)
         {
             // No windows in the current desktop? No windows at all?
             // Create a new one and load there sites...
-            loadUrl(urlList.at(0), Rekonq::NewWindow);
+            if (incognito)
+                loadUrl(urlList.at(0), Rekonq::NewPrivateWindow);
+            else
+                loadUrl(urlList.at(0), Rekonq::NewWindow);
         }
         else
         {
-            if (!ReKonfig::openExternalLinksInNewWindow())
+            if (incognito)
+            {
+                loadUrl(urlList.at(0), Rekonq::NewPrivateWindow);
+            }
+            else if (!ReKonfig::openExternalLinksInNewWindow())
             {
                 loadUrl(urlList.at(0), Rekonq::NewFocusedTab);
             }
@@ -241,8 +218,8 @@ int Application::newInstance()
                 loadUrl(urlList.at(0), Rekonq::NewWindow);
             }
 
-            if (!mainWindow()->isActiveWindow())
-                KWindowSystem::demandAttention(mainWindow()->winId(), true);
+            if (!tabWindow()->isActiveWindow())
+                KWindowSystem::demandAttention(tabWindow()->winId(), true);
         }
 
         // following arguments: what's best behavior here?
@@ -255,83 +232,103 @@ int Application::newInstance()
         else
         {
             for (int i = 1; i < urlList.count(); ++i)
-                loadUrl(urlList.at(i), Rekonq::NewWindow);
+                if (incognito)
+                    loadUrl(urlList.at(i), Rekonq::NewPrivateWindow);
+                else
+                    loadUrl(urlList.at(i), Rekonq::NewWindow);
         }
     }
-    else
+    else    // ok, NO arguments
     {
+        kDebug() << "DEFAULT MODE, NO ARGUMENTS...";
         if (isFirstLoad)
         {
-            if (hasToBeRecoveredFromCrash)
+            // NOTE: just load new tabs/windows without arguments
+            // if NOT is Session restored...
+            if (!isSessionRestored())
             {
-                loadUrl(KUrl("about:closedTabs"), Rekonq::NewWindow);
-            }
-            else
-            {
-                // NOTE: just load new tabs/windows without arguments
-                // if NOT is Session restored...
-                if (!isSessionRestored())
+                switch (ReKonfig::startupBehaviour())
                 {
-                    switch (ReKonfig::startupBehaviour())
+                case 0: // open home page
+                    if (incognito)
                     {
-                    case 0: // open home page
-                        newMainWindow()->homePage();
-                        break;
-                    case 1: // open new tab page
-                        loadUrl(KUrl("about:home"), Rekonq::NewWindow);
-                        break;
-                    case 2: // restore session
-                        if (sessionManager()->restoreSessionFromScratch())
-                        {
-                            break;
-                        }
-                    default:
-                        newMainWindow()->homePage();
+                        loadUrl(KUrl(ReKonfig::homePage()), Rekonq::NewPrivateWindow);
                         break;
                     }
+                    if (SessionManager::self()->restoreJustThePinnedTabs())
+                        loadUrl(KUrl(ReKonfig::homePage()) , Rekonq::NewTab);
+                    else
+                        loadUrl(KUrl(ReKonfig::homePage()) , Rekonq::NewWindow);
+                    break;
+                case 1: // open new tab page
+                    if (incognito)
+                    {
+                        loadUrl(KUrl("about:home"), Rekonq::NewPrivateWindow);
+                        break;
+                    }
+                    if (SessionManager::self()->restoreJustThePinnedTabs())
+                        loadUrl(KUrl("about:home") , Rekonq::NewTab);
+                    else
+                        loadUrl(KUrl("about:home"), Rekonq::NewWindow);
+                    break;
+                case 2: // restore session
+                    if (incognito)
+                    {
+                        loadUrl(KUrl("about:home"), Rekonq::NewPrivateWindow);
+                        break;
+                    }
+                    if (SessionManager::self()->restoreSessionFromScratch())
+                    {
+                        break;
+                    }
+                default:
+                    newTabWindow()->newTab();
+                    break;
                 }
             }
         }
         else
         {
+            Rekonq::OpenType type = incognito ? Rekonq::NewPrivateWindow : Rekonq::NewWindow;
+
             switch (ReKonfig::newTabsBehaviour())
             {
             case 0: // new tab page
-                loadUrl(KUrl("about:home") , Rekonq::NewWindow);
+                loadUrl(KUrl("about:home") , type);
                 break;
             case 2: // homepage
-                loadUrl(KUrl(ReKonfig::homePage()) , Rekonq::NewWindow);
+                loadUrl(KUrl(ReKonfig::homePage()) , type);
                 break;
             case 1: // blank page
             default:
-                loadUrl(KUrl("about:blank") , Rekonq::NewWindow);
+                loadUrl(KUrl("about:blank") , type);
                 break;
             }
         }
     }
 
+    // ok, now last stuffs...
     if (isFirstLoad)
     {
-        if (hasToBeRecoveredFromCrash)
+        if (hasToBeRecoveredFromCrash && !incognito)
         {
-            QTimer::singleShot(1000, mainWindow()->currentTab(), SLOT(showMessageBar()));
+            QTimer::singleShot(1000, tabWindow()->currentWebWindow(), SLOT(showCrashMessageBar()));
         }
         else
         {
-            sessionManager()->setSessionManagementEnabled(true);
+            SessionManager::self()->setSessionManagementEnabled(true);
         }
 
-        if (ReKonfig::checkDefaultSearchEngine()
-                && !hasToBeRecoveredFromCrash
-                && SearchEngine::defaultEngine().isNull())
-            QTimer::singleShot(2000, mainWindow()->currentTab(), SLOT(showSearchEngineBar()));
+        if (ReKonfig::checkDefaultSearchEngine() && !hasToBeRecoveredFromCrash && SearchEngine::defaultEngine().isNull())
+            QTimer::singleShot(2000, tabWindow()->currentWebWindow()->tabView(), SLOT(showSearchEngineBar()));
 
         // updating rekonq configuration
         updateConfiguration();
 
         setWindowIcon(KIcon("rekonq"));
 
-        historyManager();
+        // just create History Manager...
+        HistoryManager::self();
 
         ReKonfig::setRecoverOnCrash(ReKonfig::recoverOnCrash() + 1);
         saveConfiguration();
@@ -356,106 +353,26 @@ void Application::saveConfiguration() const
 }
 
 
-MainWindow *Application::mainWindow()
+TabWindow *Application::tabWindow()
 {
-    MainWindow *active = qobject_cast<MainWindow*>(QApplication::activeWindow());
+    TabWindow *active = qobject_cast<TabWindow*>(QApplication::activeWindow());
 
     if (!active)
     {
-        if (m_mainWindows.isEmpty())
+        if (m_tabWindows.isEmpty())
             return 0;
 
-        Q_FOREACH(const QWeakPointer<MainWindow> &pointer, m_mainWindows)
+        Q_FOREACH(const QWeakPointer<TabWindow> &pointer, m_tabWindows)
         {
             if (KWindowInfo(pointer.data()->effectiveWinId(), NET::WMDesktop, 0).isOnCurrentDesktop())
                 return pointer.data();
         }
-        return m_mainWindows.at(0).data();
+        return m_tabWindows.at(0).data();
     }
     return active;
 }
 
 
-HistoryManager *Application::historyManager()
-{
-    if (m_historyManager.isNull())
-    {
-        m_historyManager = new HistoryManager;
-    }
-    return m_historyManager.data();
-}
-
-
-BookmarkManager *Application::bookmarkManager()
-{
-    if (m_bookmarkManager.isNull())
-    {
-        m_bookmarkManager = new BookmarkManager;
-    }
-    return m_bookmarkManager.data();
-}
-
-
-SessionManager *Application::sessionManager()
-{
-    if (m_sessionManager.isNull())
-    {
-        m_sessionManager = new SessionManager;
-    }
-    return m_sessionManager.data();
-}
-
-
-OpenSearchManager *Application::opensearchManager()
-{
-    if (m_opensearchManager.isNull())
-    {
-        m_opensearchManager = new OpenSearchManager;
-    }
-    return m_opensearchManager.data();
-}
-
-
-IconManager *Application::iconManager()
-{
-    if (m_iconManager.isNull())
-    {
-        m_iconManager = new IconManager;
-    }
-    return m_iconManager.data();
-}
-
-
-DownloadManager *Application::downloadManager()
-{
-    if (m_downloadManager.isNull())
-    {
-        m_downloadManager = new DownloadManager(instance());
-    }
-    return m_downloadManager.data();
-}
-
-
-UserAgentManager *Application::userAgentManager()
-{
-    if (m_userAgentManager.isNull())
-    {
-        m_userAgentManager = new UserAgentManager(instance());
-    }
-    return m_userAgentManager.data();
-}
-
-
-SyncManager *Application::syncManager()
-{
-    if (m_syncManager.isNull())
-    {
-        m_syncManager = new SyncManager(instance());
-    }
-    return m_syncManager.data();
-}
-
-
 void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
 {
     if (url.isEmpty())
@@ -472,74 +389,67 @@ void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
     if (url.url().contains("about:") && url.url().contains("/"))
         newType = Rekonq::CurrentTab;
 
-    MainWindow *w = 0;
-    if (newType == Rekonq::NewWindow
-            || (newType == Rekonq::NewTab && ReKonfig::openLinksInNewWindow()))
+    TabWindow *w = 0;
+    if (newType == Rekonq::NewPrivateWindow)
     {
-        w = newMainWindow();
+        w = newTabWindow(true, true);
+        newType = Rekonq::CurrentTab;
+    }
+    else if (newType == Rekonq::NewWindow
+             || ((newType == Rekonq::NewTab || newType == Rekonq::NewFocusedTab) && tabWindowList().count() == 0))
+    {
+        w = newTabWindow();
         newType = Rekonq::CurrentTab;
     }
     else
     {
-        w = mainWindow();
+        w = tabWindow();
     }
 
-    // be SURE window exists
-    if (!w)
-        w = newMainWindow();
-
-    w->loadCheckedUrl(url, newType);
+    w->loadUrl(url, newType);
 }
 
 
-MainWindow *Application::newMainWindow(bool withTab)
+TabWindow *Application::newTabWindow(bool withTab, bool PrivateBrowsingMode)
 {
-    MainWindow *w = new MainWindow();
+    TabWindow *w = new TabWindow(withTab, PrivateBrowsingMode);
+
+    // set object name
+    int n = m_tabWindows.count() + 1;
+    w->setObjectName(QL1S("win") + QString::number(n));
+
     // This is used to track which window was activated most recently
     w->installEventFilter(this);
 
-    m_mainWindows.prepend(w);
-
-    if (withTab)
-        w->mainView()->newWebTab();    // remember using newWebTab and NOT newTab here!!
-
+    m_tabWindows.prepend(w);
     w->show();
 
     return w;
 }
 
 
-void Application::removeMainWindow(MainWindow *window)
+WebTab *Application::newWebApp()
 {
-    m_mainWindows.removeOne(window);
-    kDebug() << "Removing Window from app window list...";
+    WebTab *tab = new WebTab;
 
-    // bye bye...
-    if (m_mainWindows.count() == 0)
-        quit();
+    tab->installEventFilter(this);
+    m_webApps.prepend(tab);
+
+    tab->show();
+
+    return tab;
 }
 
 
-MainWindowList Application::mainWindowList()
+TabWindowList Application::tabWindowList()
 {
-    return m_mainWindows;
+    return m_tabWindows;
 }
 
 
-AdBlockManager *Application::adblockManager()
+WebAppList Application::webAppList()
 {
-    if (m_adblockManager.isNull())
-    {
-        m_adblockManager = new AdBlockManager;
-    }
-    return m_adblockManager.data();
-}
-
-
-void Application::newWindow()
-{
-    loadUrl(KUrl("about:home"), Rekonq::NewWindow);
-    mainWindow()->mainView()->currentUrlBar()->setFocus();
+    return m_webApps;
 }
 
 
@@ -549,18 +459,33 @@ bool Application::eventFilter(QObject* watched, QEvent* event)
     // (e.g. when another application opens a link)
     if (event->type() == QEvent::WindowActivate)
     {
-        MainWindow *window = qobject_cast<MainWindow*>(watched);
+        TabWindow *window = qobject_cast<TabWindow*>(watched);
         if (window)
         {
-            if (m_mainWindows.at(0).data() != window)
+            if (m_tabWindows.at(0).data() != window)
             {
-                int index = m_mainWindows.indexOf(QWeakPointer<MainWindow>(window));
+                int index = m_tabWindows.indexOf(QWeakPointer<TabWindow>(window));
                 Q_ASSERT(index != -1);
-                m_mainWindows.prepend(m_mainWindows.takeAt(index));
+                m_tabWindows.prepend(m_tabWindows.takeAt(index));
             }
         }
     }
 
+    // As we are filtering the events occurred to the tabwindows, check also
+    // when we close one of them, remove from tab window list and check if it was last...
+    if (event->type() == QEvent::Close)
+    {
+        TabWindow *window = qobject_cast<TabWindow*>(watched);
+        if (window)
+            m_tabWindows.removeOne(window);
+
+        WebTab *webApp = qobject_cast<WebTab*>(watched);
+        m_webApps.removeOne(webApp);
+
+        if (m_tabWindows.count() == 0 && m_webApps.count() == 0)
+            quit();
+    }
+
     return QObject::eventFilter(watched, event);
 }
 
@@ -569,19 +494,15 @@ void Application::updateConfiguration()
 {
     // ============== Tabs ==================
     bool b = ReKonfig::closeTabSelectPrevious();
-    Q_FOREACH(const QWeakPointer<MainWindow> &w, m_mainWindows)
+    Q_FOREACH(const QWeakPointer<TabWindow> &w, m_tabWindows)
     {
-        MainView *mv = w.data()->mainView();
-        mv->updateTabBarVisibility();
-
-        mv->tabBar()->setAnimatedTabHighlighting(ReKonfig::animatedTabHighlighting());
-
         if (b)
-            mv->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
+            w.data()->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
         else
-            mv->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectRightTab);
+            w.data()->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectRightTab);
     }
 
+
     QWebSettings *defaultSettings = QWebSettings::globalSettings();
 
     // =========== Fonts ==============
@@ -595,9 +516,9 @@ void Application::updateConfiguration()
     // compute font size
     // (I have to admit I know nothing about these DPI questions..: copied from kwebkitpart, as someone suggested)
     // font size in pixels =  font size in inches × screen dpi
-    if (mainWindow() && mainWindow()->currentTab())
+    if (tabWindow() && tabWindow()->currentWebWindow())
     {
-        int logDpiY = mainWindow()->currentTab()->view()->logicalDpiY();
+        int logDpiY = tabWindow()->currentWebWindow()->logicalDpiY();
         float toPix = (logDpiY < 96.0)
                       ? 96.0 / 72.0
                       : logDpiY / 72.0 ;
@@ -651,40 +572,41 @@ void Application::updateConfiguration()
     defaultSettings->setUserStyleSheetUrl(ReKonfig::userCSS());
 
     // ====== load Settings on main classes
-    historyManager()->loadSettings();
+    HistoryManager::self()->loadSettings();
 
     defaultSettings = 0;
 
-    if (!mainWindow())
+    if (!tabWindow())
         return;
 
-    ReKonfig::useFavicon()
-    ? mainWindow()->changeWindowIcon(mainWindow()->mainView()->currentIndex())
-    : mainWindow()->setWindowIcon(KIcon("rekonq"))
-    ;
-
+    // FIXME What about this?
+//     ReKonfig::useFavicon()
+//     ? tabWindow()->changeWindowIcon(tabWindow()->mainView()->currentIndex())
+//     : tabWindow()->setWindowIcon(KIcon("rekonq"))
+//     ;
+//
     // hovering unfocused tabs options
     switch (ReKonfig::hoveringTabOption())
     {
     case 0: // tab previews
     case 3: // nothing
-        for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
+        for (int i = 0; i < tabWindow()->tabBar()->count(); i++)
         {
-            mainWindow()->mainView()->tabBar()->setTabToolTip(i, QL1S(""));
+            tabWindow()->tabBar()->setTabToolTip(i, QL1S(""));
         }
         break;
 
     case 1: // title previews
-        for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
+        for (int i = 0; i < tabWindow()->tabBar()->count(); i++)
         {
-            mainWindow()->mainView()->tabBar()->setTabToolTip(i, mainWindow()->mainView()->tabText(i).remove('&'));
+            tabWindow()->tabBar()->setTabToolTip(i, tabWindow()->tabText(i).remove('&'));
         }
         break;
 
     case 2: // url previews
-        for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
+        for (int i = 0; i < tabWindow()->tabBar()->count(); i++)
         {
-            mainWindow()->mainView()->tabBar()->setTabToolTip(i, mainWindow()->mainView()->webTab(i)->url().toMimeDataString());
+            tabWindow()->tabBar()->setTabToolTip(i, tabWindow()->webWindow(i)->url().toMimeDataString());
         }
         break;
 
@@ -696,80 +618,18 @@ void Application::updateConfiguration()
 }
 
 
-void Application::setPrivateBrowsingMode(bool b)
-{
-// NOTE
-// to let work nicely Private Browsing, we need the following:
-// - enable WebKit Private Browsing mode :)
-// - treat all cookies as session cookies
-//  (so that they do not get saved to a persistent storage). Available from KDE SC 4.5.72, see BUG: 250122
-// - favicons (fixed in rekonq 0.5.87)
-// - save actual session (to restore it when Private Mode is closed) and stop storing it
-// - disable history saving
-
-    QWebSettings *settings = QWebSettings::globalSettings();
-    bool isJustEnabled = settings->testAttribute(QWebSettings::PrivateBrowsingEnabled);
-    if (isJustEnabled == b)
-        return;     // uhm... something goes wrong...
-
-    if (b)
-    {
-        QString caption = i18n("Are you sure you want to turn on private browsing?");
-        QString text = i18n("<b>%1</b>"
-                            "<p>rekonq will save your current tabs for when you'll stop private browsing the net.</p>", caption);
-
-        int button = KMessageBox::warningContinueCancel(mainWindow(),
-                     text, caption, KStandardGuiItem::cont(), KStandardGuiItem::cancel(),
-                     i18n("don't ask again"));
-        if (button != KMessageBox::Continue)
-        {
-            // The user canceled so we should uncheck the box
-            _privateBrowsingAction->setChecked(false);
-            return;
-        }
-
-        sessionManager()->setSessionManagementEnabled(false);
-        settings->setAttribute(QWebSettings::PrivateBrowsingEnabled, true);
-        _privateBrowsingAction->setChecked(true);
-
-        loadUrl(KUrl("about:home"), Rekonq::NewWindow);
-
-        MainWindow *activeOne = m_mainWindows.at(0).data();
-
-        Q_FOREACH(const QWeakPointer<MainWindow> &w, m_mainWindows)
-        {
-            if (w.data() != activeOne)
-                w.data()->close();
-        }
-    }
-    else
-    {
-        settings->setAttribute(QWebSettings::PrivateBrowsingEnabled, false);
-        _privateBrowsingAction->setChecked(false);
-
-        int newWindows = sessionManager()->restoreSavedSession();
-        if (newWindows == 0)
-        {
-            loadUrl(KUrl("about:home"), Rekonq::NewWindow);
-            newWindows++;
-        }
-
-        for (int i = newWindows; i < m_mainWindows.count(); ++i)
-        {
-            m_mainWindows.at(i).data()->close();
-        }
-
-        sessionManager()->setSessionManagementEnabled(true);
-    }
-}
-
-
 void Application::queryQuit()
 {
-    if (mainWindowList().count() > 1)
+    if (m_webApps.count() > 0)
+    {
+        tabWindow()->close();
+        return;
+    }
+
+    if (tabWindowList().count() > 1)
     {
         int answer = KMessageBox::questionYesNoCancel(
-                         mainWindow(),
+                         tabWindow(),
                          i18n("Do you want to close the window or the whole application?"),
                          i18n("Application/Window closing..."),
                          KGuiItem(i18n("C&lose Current Window"),
@@ -782,7 +642,7 @@ void Application::queryQuit()
         switch (answer)
         {
         case KMessageBox::Yes:
-            mainWindow()->close();
+            tabWindow()->close();
             return;
 
         case KMessageBox::No:
@@ -798,24 +658,101 @@ void Application::queryQuit()
 }
 
 
+void Application::clearPrivateData()
+{
+    QPointer<KDialog> dialog = new KDialog(tabWindow());
+    dialog->setCaption(i18nc("@title:window", "Clear Private Data"));
+    dialog->setButtons(KDialog::Ok | KDialog::Cancel);
+
+    dialog->button(KDialog::Ok)->setIcon(KIcon("edit-clear"));
+    dialog->button(KDialog::Ok)->setText(i18n("Clear"));
+
+    Ui::ClearDataWidget clearWidget;
+    QWidget widget;
+    clearWidget.setupUi(&widget);
+    clearWidget.clearHistory->setChecked(ReKonfig::clearHistory());
+    clearWidget.clearDownloads->setChecked(ReKonfig::clearDownloads());
+    clearWidget.clearCookies->setChecked(ReKonfig::clearCookies());
+    clearWidget.clearCachedPages->setChecked(ReKonfig::clearCachedPages());
+    clearWidget.clearWebIcons->setChecked(ReKonfig::clearWebIcons());
+    clearWidget.homePageThumbs->setChecked(ReKonfig::clearHomePageThumbs());
+
+    dialog->setMainWidget(&widget);
+    dialog->exec();
+
+    if (dialog->result() == QDialog::Accepted)
+    {
+        //Save current state
+        ReKonfig::setClearHistory(clearWidget.clearHistory->isChecked());
+        ReKonfig::setClearDownloads(clearWidget.clearDownloads->isChecked());
+        ReKonfig::setClearCookies(clearWidget.clearDownloads->isChecked());
+        ReKonfig::setClearCachedPages(clearWidget.clearCachedPages->isChecked());
+        ReKonfig::setClearWebIcons(clearWidget.clearWebIcons->isChecked());
+        ReKonfig::setClearHomePageThumbs(clearWidget.homePageThumbs->isChecked());
+
+        if (clearWidget.clearHistory->isChecked())
+        {
+            HistoryManager::self()->clear();
+        }
+
+        if (clearWidget.clearDownloads->isChecked())
+        {
+            DownloadManager::self()->clearDownloadsHistory();
+        }
+
+        if (clearWidget.clearCookies->isChecked())
+        {
+            QDBusInterface kcookiejar("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer");
+            QDBusReply<void> reply = kcookiejar.call("deleteAllCookies");
+        }
+
+        if (clearWidget.clearCachedPages->isChecked())
+        {
+            KProcess::startDetached(KStandardDirs::findExe("kio_http_cache_cleaner"),
+                                    QStringList(QL1S("--clear-all")));
+        }
+
+        if (clearWidget.clearWebIcons->isChecked())
+        {
+            IconManager::self()->clearIconCache();
+        }
+
+        if (clearWidget.homePageThumbs->isChecked())
+        {
+            QString path = KStandardDirs::locateLocal("cache", QString("thumbs/rekonq"), true);
+            path.remove("rekonq");
+            QDir cacheDir(path);
+            QStringList fileList = cacheDir.entryList();
+            Q_FOREACH(const QString & str, fileList)
+            {
+                QFile file(path + str);
+                file.remove();
+            }
+        }
+    }
+
+    dialog->deleteLater();
+}
+
+
 void Application::createWebAppShortcut()
 {
-    KUrl u = mainWindow()->currentTab()->url();
+    KUrl u = tabWindow()->currentWebWindow()->url();
     QString h = u.host();
 
-    QPointer<KDialog> dialog = new KDialog(mainWindow());
+    QPointer<KDialog> dialog = new KDialog(tabWindow());
     dialog->setCaption(i18nc("@title:window", "Create Application Shortcut"));
     dialog->setButtons(KDialog::Ok | KDialog::Cancel);
     dialog->button(KDialog::Ok)->setText(i18n("Create"));
     dialog->setMinimumSize(400, 50);
-    dialog->setWindowIcon(QIcon(iconManager()->iconForUrl(u).pixmap(16)));
+    dialog->setWindowIcon(QIcon(IconManager::self()->iconForUrl(u).pixmap(16)));
 
     Ui::webAppCreation wAppWidget;
     QWidget widget;
     wAppWidget.setupUi(&widget);
 
-    const QString title = mainWindow()->currentTab()->view()->title().remove('&');
-    wAppWidget.nameLineEdit->setText(title);
+    QString webAppTitle = tabWindow()->currentWebWindow()->title().remove('&');
+    wAppWidget.nameLineEdit->setText(webAppTitle);
     wAppWidget.kcfg_createDesktopAppShortcut->setChecked(ReKonfig::createDesktopAppShortcut());
     wAppWidget.kcfg_createMenuAppShortcut->setChecked(ReKonfig::createMenuAppShortcut());
 
@@ -827,15 +764,24 @@ void Application::createWebAppShortcut()
         ReKonfig::setCreateDesktopAppShortcut(wAppWidget.kcfg_createDesktopAppShortcut->isChecked());
         ReKonfig::setCreateMenuAppShortcut(wAppWidget.kcfg_createMenuAppShortcut->isChecked());
 
-        iconManager()->saveDesktopIconForUrl(u);
+        IconManager::self()->saveDesktopIconForUrl(u);
         QString iconPath = KStandardDirs::locateLocal("cache" , "favicons/" , true) + h + QL1S("_WEBAPPICON.png");
 
+        if (!wAppWidget.nameLineEdit->text().isEmpty())
+            webAppTitle = wAppWidget.nameLineEdit->text();
+
+        QString webAppDescription;
+        if (!wAppWidget.descriptionLineEdit->text().isEmpty())
+            webAppDescription = wAppWidget.descriptionLineEdit->text();
+
         QString shortcutString = QL1S("#!/usr/bin/env xdg-open\n")
                                  + QL1S("[Desktop Entry]\n")
-                                 + QL1S("Name=") + (wAppWidget.nameLineEdit->text().isEmpty() ? QL1S("kwebapp") : wAppWidget.nameLineEdit->text()) + QL1S("\n")
-                                 + QL1S("GenericName=") + (wAppWidget.descriptionLineEdit->text().isEmpty() ? QL1S("") : wAppWidget.descriptionLineEdit->text()) + QL1S("\n")
+                                 + QL1S("Name=") + webAppTitle
+                                 + QL1S("\n")
+                                 + QL1S("GenericName=") + webAppDescription
+                                 + QL1S("\n")
                                  + QL1S("Icon=") + iconPath + QL1S("\n")
-                                 + QL1S("Exec=kwebapp ") + u.url() + QL1S("\n")
+                                 + QL1S("Exec=rekonq --webapp ") + u.url() + QL1S("\n")
                                  + QL1S("Type=Application\n")
                                  + QL1S("Categories=Application;Network\n")
                                  ;
@@ -843,7 +789,7 @@ void Application::createWebAppShortcut()
         if (ReKonfig::createDesktopAppShortcut())
         {
             QString desktop = KGlobalSettings::desktopPath();
-            QFile wAppFile(desktop + QL1C('/') + title);
+            QFile wAppFile(desktop + QL1C('/') + webAppTitle);
 
             if (!wAppFile.open(QIODevice::WriteOnly | QIODevice::Text))
             {
@@ -862,7 +808,7 @@ void Application::createWebAppShortcut()
         if (ReKonfig::createMenuAppShortcut())
         {
             QString appMenuDir = KStandardDirs::locateLocal("xdgdata-apps", QString());
-            QFile wAppFile(appMenuDir + QL1C('/') + title + QL1S(".desktop"));
+            QFile wAppFile(appMenuDir + QL1C('/') + webAppTitle + QL1S(".desktop"));
 
             if (!wAppFile.open(QIODevice::WriteOnly | QIODevice::Text))
             {
@@ -881,3 +827,44 @@ void Application::createWebAppShortcut()
 
     dialog->deleteLater();
 }
+
+
+void Application::bookmarksToolbarToggled(bool b)
+{
+    emit toggleBookmarksToolbar(b);
+}
+
+
+void Application::newPrivateBrowsingWindow()
+{
+    // NOTE: what about an "about:incognito" page?
+    loadUrl(KUrl("about:home"), Rekonq::NewPrivateWindow);
+}
+
+
+void Application::pageCreated(WebPage *pg)
+{
+    if (m_tabWindows.isEmpty())
+    {
+        // NOTE: This is "adjusted" from newTabWindow() code...
+        TabWindow *w = new TabWindow(pg);
+
+        // set object name
+        int n = m_tabWindows.count() + 1;
+        w->setObjectName(QL1S("win") + QString::number(n));
+
+        // This is used to track which window was activated most recently
+        w->installEventFilter(this);
+
+        m_tabWindows.prepend(w);
+        w->show();
+
+        return;
+    }
+
+    TabWindow *tw = tabWindow();
+    tw->newTab(pg);
+
+    tw->activateWindow();
+    tw->raise();
+}
