@@ -2,9 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
-* Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2012 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -33,35 +31,22 @@
 // Rekonq Includes
 #include "rekonq_defines.h"
 
-// Local Includes
-#include "downloaditem.h"
-
 // KDE Includes
 #include <KUniqueApplication>
+#include <KUrl>
 
 // Qt Includes
 #include <QWeakPointer>
 
 // Forward Declarations
-class KAction;
+class TabWindow;
+class WebWindow;
 
-class QWebHistory;
-
-class AdBlockManager;
-class BookmarkManager;
-class DownloadManager;
-class HistoryManager;
-class IconManager;
-class MainWindow;
-class OpenSearchManager;
-class SessionManager;
-class UserAgentManager;
-class SyncManager;
 class WebTab;
+class WebPage;
 
-
-typedef QList< QWeakPointer<MainWindow> > MainWindowList;
-
+typedef QList< QWeakPointer<TabWindow> > TabWindowList;
+typedef QList<WebTab *> WebAppList;
 
 // ---------------------------------------------------------------------------------------------------------------
 
@@ -71,7 +56,7 @@ typedef QList< QWeakPointer<MainWindow> > MainWindowList;
 /**
   * Rekonq Application class
   */
-class REKONQ_TESTS_EXPORT Application : public KUniqueApplication
+class Application : public KUniqueApplication
 {
     Q_OBJECT
 
@@ -82,24 +67,12 @@ public:
     int newInstance();
     static Application *instance();
 
-    MainWindow *mainWindow();
-    MainWindow *newMainWindow(bool withTab = true);
-    MainWindowList mainWindowList();
+    TabWindow *tabWindow();
+    TabWindowList tabWindowList();
 
-    HistoryManager *historyManager();
-    BookmarkManager *bookmarkManager();
-    SessionManager *sessionManager();
-    AdBlockManager *adblockManager();
-    OpenSearchManager *opensearchManager();
-    IconManager *iconManager();
-    DownloadManager *downloadManager();
-    UserAgentManager *userAgentManager();
-    SyncManager *syncManager();
-
-    KAction *privateBrowsingAction()
-    {
-        return _privateBrowsingAction;
-    };
+    WebAppList webAppList();
+    
+    void bookmarksToolbarToggled(bool);
 
 public Q_SLOTS:
     /**
@@ -119,37 +92,34 @@ public Q_SLOTS:
                  const Rekonq::OpenType& type = Rekonq::CurrentTab
                 );
 
-    void newWindow();
-    void removeMainWindow(MainWindow *window);
+    TabWindow *newTabWindow(bool withTab = true, bool PrivateBrowsingMode = false);
+
+    WebTab *newWebApp();
 
 protected:
     // This is used to track which window was activated most recently
     bool eventFilter(QObject *watched, QEvent *event);
 
+Q_SIGNALS:
+    void toggleBookmarksToolbar(bool);
+
 private Q_SLOTS:
     void updateConfiguration();
 
-    // the general place to set private browsing
-    void setPrivateBrowsingMode(bool);
+    // clear private data
+    void clearPrivateData();
 
     void queryQuit();
 
     void createWebAppShortcut();
 
+    void newPrivateBrowsingWindow();
+
+    void pageCreated(WebPage *);
+
 private:
-    QWeakPointer<HistoryManager> m_historyManager;
-    QWeakPointer<BookmarkManager> m_bookmarkManager;
-    QWeakPointer<SessionManager> m_sessionManager;
-    QWeakPointer<AdBlockManager> m_adblockManager;
-    QWeakPointer<OpenSearchManager> m_opensearchManager;
-    QWeakPointer<IconManager> m_iconManager;
-    QWeakPointer<DownloadManager> m_downloadManager;
-    QWeakPointer<UserAgentManager> m_userAgentManager;
-    QWeakPointer<SyncManager> m_syncManager;
-
-    MainWindowList m_mainWindows;
-
-    KAction *_privateBrowsingAction;
+    TabWindowList m_tabWindows;
+    WebAppList m_webApps;
 };
 
 #endif // APPLICATION_H
