@@ -33,12 +33,9 @@
 
 // Local Includes
 #include "application.h"
-#include "bookmarkspanel.h"
 #include "bookmarkstoolbar.h"
 #include "bookmarkowner.h"
 #include "iconmanager.h"
-#include "mainwindow.h"
-#include "webtab.h"
 
 // KDE Includes
 #include <KActionCollection>
@@ -48,6 +45,25 @@
 #include <QtCore/QFile>
 
 
+// ----------------------------------------------------------------------------------------------
+
+
+QWeakPointer<BookmarkManager> BookmarkManager::s_bookmarkManager;
+
+
+BookmarkManager *BookmarkManager::self()
+{
+    if (s_bookmarkManager.isNull())
+    {
+        s_bookmarkManager = new BookmarkManager(qApp);
+    }
+    return s_bookmarkManager.data();
+}
+
+
+// ----------------------------------------------------------------------------------------------
+
+
 BookmarkManager::BookmarkManager(QObject *parent)
     : QObject(parent)
     , m_manager(0)
@@ -69,15 +85,14 @@ BookmarkManager::BookmarkManager(QObject *parent)
         delete tempManager;
     }
 
-    connect(m_manager, SIGNAL(changed(QString, QString)), this, SLOT(slotBookmarksChanged()));
+    connect(m_manager, SIGNAL(changed(QString,QString)), this, SLOT(slotBookmarksChanged()));
 
     // setup menu
     m_owner = new BookmarkOwner(m_manager, this);
-    connect(m_owner, SIGNAL(openUrl(KUrl, Rekonq::OpenType)),
-            this, SIGNAL(openUrl(KUrl, Rekonq::OpenType)));
+    connect(m_owner, SIGNAL(openUrl(KUrl,Rekonq::OpenType)), this, SIGNAL(openUrl(KUrl,Rekonq::OpenType)));
 
     // bookmarks loading
-    connect(this, SIGNAL(openUrl(KUrl, Rekonq::OpenType)), rApp, SLOT(loadUrl(KUrl, Rekonq::OpenType)));
+    connect(this, SIGNAL(openUrl(KUrl,Rekonq::OpenType)), rApp, SLOT(loadUrl(KUrl,Rekonq::OpenType)));
 }
 
 
@@ -87,19 +102,6 @@ BookmarkManager::~BookmarkManager()
 }
 
 
-KActionMenu* BookmarkManager::bookmarkActionMenu(QWidget *parent)
-{
-    KMenu *menu = new KMenu(parent);
-    KActionMenu *bookmarkActionMenu = new KActionMenu(menu);
-    bookmarkActionMenu->setMenu(menu);
-    bookmarkActionMenu->setText(i18n("&Bookmarks"));
-    BookmarkMenu *bMenu = new BookmarkMenu(m_manager, m_owner, menu, m_actionCollection);
-    bMenu->setParent(menu);
-
-    return bookmarkActionMenu;
-}
-
-
 void BookmarkManager::registerBookmarkBar(BookmarkToolBar *toolbar)
 {
     if (m_bookmarkToolBars.contains(toolbar))
@@ -115,29 +117,6 @@ void BookmarkManager::removeBookmarkBar(BookmarkToolBar *toolbar)
 }
 
 
-void BookmarkManager::registerBookmarkPanel(BookmarksPanel *panel)
-{
-    if (panel && !m_bookmarkPanels.contains(panel))
-    {
-        m_bookmarkPanels.append(panel);
-        connect(panel, SIGNAL(expansionChanged()), this, SLOT(slotPanelChanged()));
-    }
-}
-
-
-void BookmarkManager::removeBookmarkPanel(BookmarksPanel *panel)
-{
-    if (!panel)
-        return;
-
-    m_bookmarkPanels.removeOne(panel);
-    panel->disconnect(this);
-
-    if (m_bookmarkPanels.isEmpty())
-        emitChanged();
-}
-
-
 QAction* BookmarkManager::actionByName(const QString &name)
 {
     QAction *action = m_actionCollection->action(name);
@@ -182,22 +161,22 @@ void BookmarkManager::slotBookmarksChanged()
     {
         if (bookmarkToolBar)
         {
-            bookmarkToolBar->toolBar()->clear();
+            bookmarkToolBar->clear();
             fillBookmarkBar(bookmarkToolBar);
         }
     }
 
-    // this update about:bookmarks page...
-    if (rApp->mainWindow()
-            && rApp->mainWindow()->currentTab()
-            && rApp->mainWindow()->currentTab()->url().toMimeDataString().contains("about:bookmarks")
-       )
-        rApp->loadUrl(KUrl("about:bookmarks"), Rekonq::CurrentTab);
-
+    // NOTE with this signal, we should (eventual) update about:bookmarks page...
     emit bookmarksUpdated();
 }
 
 
+KBookmark BookmarkManager::bookmarkCurrentPage(const KBookmark &bookmark)
+{
+    return m_owner->bookmarkCurrentPage(bookmark);
+}
+
+
 void BookmarkManager::fillBookmarkBar(BookmarkToolBar *toolBar)
 {
     KBookmarkGroup root = m_manager->toolbar();
@@ -208,7 +187,7 @@ void BookmarkManager::fillBookmarkBar(BookmarkToolBar *toolBar)
     {
         if (bookmark.isGroup())
         {
-            KBookmarkActionMenu *menuAction = new KBookmarkActionMenu(bookmark.toGroup(), this);
+            KBookmarkActionMenu *menuAction = new KBookmarkActionMenu(bookmark.toGroup(), toolBar);
             menuAction->setDelayed(false);
             BookmarkMenu *bMenu = new BookmarkMenu(m_manager, m_owner, menuAction->menu(), bookmark.address());
             bMenu->setParent(menuAction->menu());
@@ -216,40 +195,24 @@ void BookmarkManager::fillBookmarkBar(BookmarkToolBar *toolBar)
             connect(menuAction->menu(), SIGNAL(aboutToShow()), toolBar, SLOT(menuDisplayed()));
             connect(menuAction->menu(), SIGNAL(aboutToHide()), toolBar, SLOT(menuHidden()));
 
-            toolBar->toolBar()->addAction(menuAction);
-            toolBar->toolBar()->widgetForAction(menuAction)->installEventFilter(toolBar);
+            toolBar->addAction(menuAction);
+            toolBar->widgetForAction(menuAction)->installEventFilter(toolBar);
         }
         else if (bookmark.isSeparator())
         {
-            toolBar->toolBar()->addSeparator();
+            toolBar->addSeparator();
         }
         else
         {
-            KBookmarkAction *action = new KBookmarkAction(bookmark, m_owner, this);
-            action->setIcon(rApp->iconManager()->iconForUrl(KUrl(bookmark.url())));
-            connect(action, SIGNAL(hovered()), toolBar, SLOT(actionHovered()));
-            toolBar->toolBar()->addAction(action);
-            toolBar->toolBar()->widgetForAction(action)->installEventFilter(toolBar);
+            KBookmarkAction *action = new KBookmarkAction(bookmark, m_owner, toolBar);
+            action->setIcon(IconManager::self()->iconForUrl(KUrl(bookmark.url())));
+            toolBar->addAction(action);
+            toolBar->widgetForAction(action)->installEventFilter(toolBar);
         }
     }
 }
 
 
-void BookmarkManager::slotPanelChanged()
-{
-    Q_FOREACH(BookmarksPanel * panel, m_bookmarkPanels)
-    {
-        if (panel && panel != sender())
-            panel->loadFoldedState();
-    }
-    if (rApp->mainWindow()
-            && rApp->mainWindow()->currentTab()
-            && rApp->mainWindow()->currentTab()->url().toMimeDataString().contains("about:bookmarks")
-       )
-        rApp->loadUrl(KUrl("about:bookmarks"), Rekonq::CurrentTab);
-}
-
-
 void BookmarkManager::find(QList<KBookmark> *list, const KBookmark &bookmark, const QString &text)
 {
     if (bookmark.isGroup())
