@@ -39,16 +39,15 @@
 
 // Qt Includes
 #include <QObject>
+#include <QWeakPointer>
 
 // Forward Declarations
-class BookmarksPanel;
 class BookmarkToolBar;
 class BookmarkOwner;
 class BookmarkMenu;
 
 class KAction;
 class KActionCollection;
-class KActionMenu;
 class KBookmarkGroup;
 class KBookmarkManager;
 class KUrl;
@@ -68,20 +67,13 @@ class BookmarkManager : public QObject
 
 public:
     /**
-    * @short Class constructor.
-    * Connect BookmarksProvider with bookmarks source
-    * (actually konqueror's bookmarks).
-    * @param parent The MainWindow to provide bookmarks objects.
-    */
-    BookmarkManager(QObject *parent = 0);
-    ~BookmarkManager();
-
-    /**
-     * @short Get the Bookmarks Menu Action
-     * @param the parent widget
-     * @return the Bookmarks Menu
+     * Entry point.
+     * Access to BookmarkManager class by using
+     * BookmarkManager::self()->thePublicMethodYouNeed()
      */
-    KActionMenu* bookmarkActionMenu(QWidget *parent);
+    static BookmarkManager *self();
+
+    ~BookmarkManager();
 
     /**
     * @short set the Bookmarks Toolbar Action
@@ -89,9 +81,6 @@ public:
     void registerBookmarkBar(BookmarkToolBar *toolbar);
     void removeBookmarkBar(BookmarkToolBar *toolbar);
 
-    void registerBookmarkPanel(BookmarksPanel *panel);
-    void removeBookmarkPanel(BookmarksPanel *panel);
-
     /**
      * @short Get action by name
      * This method returns poiner bookmark action of given name.
@@ -123,7 +112,9 @@ public:
     KBookmark bookmarkForUrl(const KUrl &url);
 
     KBookmark findByAddress(const QString &);
+
     void openFolderinTabs(const KBookmarkGroup &bm);
+
     void emitChanged();
 
     static inline QString bookmark_mime_type()
@@ -131,6 +122,15 @@ public:
         return QL1S("application/x-rekonq-bookmark");
     }
 
+private:
+    /**
+    * @short Class constructor.
+    * Connect BookmarksProvider with bookmarks source
+    * (actually konqueror's bookmarks).
+    * @param parent The WebWindow to provide bookmarks objects.
+    */
+    explicit BookmarkManager(QObject *parent = 0);
+
 public Q_SLOTS:
     /**
      * @short Waits for signal that the group with the address has been modified by the caller.
@@ -145,8 +145,7 @@ public Q_SLOTS:
 
     void slotEditBookmarks();
 
-private Q_SLOTS:
-    void slotPanelChanged();
+    KBookmark bookmarkCurrentPage(const KBookmark &bookmark = KBookmark());
 
 Q_SIGNALS:
     /**
@@ -165,7 +164,8 @@ private:
     BookmarkOwner *m_owner;
     KActionCollection *m_actionCollection;
     QList<BookmarkToolBar *> m_bookmarkToolBars;
-    QList<BookmarksPanel *> m_bookmarkPanels;
+
+    static QWeakPointer<BookmarkManager> s_bookmarkManager;
 };
 
 
