@@ -30,17 +30,17 @@
 #include "bookmarkstoolbar.moc"
 
 // Local Includes
-#include "iconmanager.h"
 #include "bookmarkscontextmenu.h"
-#include "mainwindow.h"
-#include "application.h"
 #include "bookmarkmanager.h"
 #include "bookmarkowner.h"
-#include "webtab.h"
+
+#include "iconmanager.h"
+#include "webwindow.h"
 
 // Qt Includes
-#include <QtGui/QFrame>
-#include <QtGui/QActionEvent>
+#include <QFrame>
+#include <QActionEvent>
+#include <QApplication>
 
 
 BookmarkMenu::BookmarkMenu(KBookmarkManager *manager,
@@ -93,7 +93,7 @@ QAction * BookmarkMenu::actionForBookmark(const KBookmark &bookmark)
     else
     {
         KBookmarkAction *action = new KBookmarkAction(bookmark, owner(), this);
-        action->setIcon(rApp->iconManager()->iconForUrl(KUrl(bookmark.url())));
+        action->setIcon(IconManager::self()->iconForUrl(KUrl(bookmark.url())));
         connect(action, SIGNAL(hovered()), this, SLOT(actionHovered()));
         return action;
     }
@@ -136,57 +136,50 @@ void BookmarkMenu::addOpenFolderInTabs()
 
         if (!bookmark.isNull())
         {
-            parentMenu()->addAction(rApp->bookmarkManager()->owner()->createAction(group, BookmarkOwner::OPEN_FOLDER));
+            parentMenu()->addAction(BookmarkManager::self()->owner()->createAction(group, BookmarkOwner::OPEN_FOLDER));
         }
     }
 }
 
 
-void BookmarkMenu::actionHovered()
-{
-    KBookmarkActionInterface* action = dynamic_cast<KBookmarkActionInterface *>(sender());
-    if (action)
-        rApp->mainWindow()->notifyMessage(action->bookmark().url().url());
-}
-
-
 // ------------------------------------------------------------------------------------------------------
 
 
-BookmarkToolBar::BookmarkToolBar(KToolBar *toolBar, QObject *parent)
-    : QObject(parent)
-    , m_toolBar(toolBar)
+BookmarkToolBar::BookmarkToolBar(QWidget *parent)
+    : KToolBar(parent, false, false)
     , m_currentMenu(0)
     , m_dragAction(0)
     , m_dropAction(0)
     , m_checkedAction(0)
     , m_filled(false)
 {
-    toolBar->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(toolBar, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(contextMenu(QPoint)));
-    connect(rApp->bookmarkManager()->manager(), SIGNAL(changed(QString, QString)), this, SLOT(hideMenu()));
-    toolBar->setAcceptDrops(true);
-    toolBar->installEventFilter(this);
-    toolBar->setShortcutEnabled(false);
+    setContextMenuPolicy(Qt::CustomContextMenu);
 
-    if (toolBar->isVisible())
+    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(contextMenu(QPoint)));
+    connect(BookmarkManager::self()->manager(), SIGNAL(changed(QString,QString)), this, SLOT(hideMenu()));
+
+    setAcceptDrops(true);
+    installEventFilter(this);
+    setShortcutEnabled(false);
+
+    if (isVisible())
     {
-        rApp->bookmarkManager()->fillBookmarkBar(this);
+        BookmarkManager::self()->fillBookmarkBar(this);
         m_filled = true;
     }
 }
 
 
-KToolBar* BookmarkToolBar::toolBar()
+BookmarkToolBar::~BookmarkToolBar()
 {
-    return m_toolBar;
+    clear();
 }
 
 
 void BookmarkToolBar::contextMenu(const QPoint &point)
 {
-    KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface*>(toolBar()->actionAt(point));
-    KBookmark bookmark = rApp->bookmarkManager()->manager()->toolbar();
+    KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface*>(actionAt(point));
+    KBookmark bookmark = BookmarkManager::self()->manager()->toolbar();
     bool nullAction = true;
     if (action)
     {
@@ -195,10 +188,11 @@ void BookmarkToolBar::contextMenu(const QPoint &point)
     }
 
     BookmarksContextMenu menu(bookmark,
-                              rApp->bookmarkManager()->manager(),
-                              rApp->bookmarkManager()->owner(),
+                              BookmarkManager::self()->manager(),
+                              BookmarkManager::self()->owner(),
                               nullAction);
-    menu.exec(toolBar()->mapToGlobal(point));
+
+    menu.exec(mapToGlobal(point));
 }
 
 
@@ -230,13 +224,13 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
     {
         // To switch root folders as in a menubar
 
-        KBookmarkActionMenu* act = dynamic_cast<KBookmarkActionMenu *>(toolBar()->actionAt(toolBar()->mapFromGlobal(QCursor::pos())));
+        KBookmarkActionMenu* act = dynamic_cast<KBookmarkActionMenu *>(actionAt(mapFromGlobal(QCursor::pos())));
 
         if (event->type() == QEvent::MouseMove && act && act->menu() != m_currentMenu)
         {
             m_currentMenu->hide();
-            QPoint pos = toolBar()->mapToGlobal(toolBar()->widgetForAction(act)->pos());
-            act->menu()->popup(QPoint(pos.x(), pos.y() + toolBar()->widgetForAction(act)->height()));
+            QPoint pos = mapToGlobal(widgetForAction(act)->pos());
+            act->menu()->popup(QPoint(pos.x(), pos.y() + widgetForAction(act)->height()));
         }
         else if (event->type() == QEvent::MouseButtonPress && act)
         {
@@ -252,7 +246,7 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
     {
         if (!m_filled)
         {
-            rApp->bookmarkManager()->fillBookmarkBar(this);
+            BookmarkManager::self()->fillBookmarkBar(this);
             m_filled = true;
         }
     }
@@ -263,7 +257,7 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
         QActionEvent *actionEvent = static_cast<QActionEvent*>(event);
         if (actionEvent && actionEvent->action() != m_dropAction)
         {
-            QWidget *widget = toolBar()->widgetForAction(actionEvent->action());
+            QWidget *widget = widgetForAction(actionEvent->action());
             if (widget)
             {
                 widget->removeEventFilter(this);
@@ -277,7 +271,7 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
         QActionEvent *actionEvent = static_cast<QActionEvent*>(event);
         if (actionEvent && actionEvent->action() != m_dropAction)
         {
-            QWidget *widget = toolBar()->widgetForAction(actionEvent->action());
+            QWidget *widget = widgetForAction(actionEvent->action());
             if (widget)
             {
                 widget->removeEventFilter(this);
@@ -293,9 +287,9 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                 || dragEvent->mimeData()->hasFormat("text/uri-list")
                 || dragEvent->mimeData()->hasFormat("text/plain"))
         {
-            QFrame* dropIndicatorWidget = new QFrame(toolBar());
+            QFrame* dropIndicatorWidget = new QFrame(this);
             dropIndicatorWidget->setFrameShape(QFrame::VLine);
-            m_dropAction = toolBar()->insertWidget(toolBar()->actionAt(dragEvent->pos()), dropIndicatorWidget);
+            m_dropAction = insertWidget(actionAt(dragEvent->pos()), dropIndicatorWidget);
 
             dragEvent->accept();
         }
@@ -325,13 +319,13 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                 || dragEvent->mimeData()->hasFormat("text/uri-list")
                 || dragEvent->mimeData()->hasFormat("text/plain"))
         {
-            QAction *overAction = toolBar()->actionAt(dragEvent->pos());
+            QAction *overAction = actionAt(dragEvent->pos());
             KBookmarkActionInterface *overActionBK = dynamic_cast<KBookmarkActionInterface*>(overAction);
-            QWidget *widgetAction = toolBar()->widgetForAction(overAction);
+            QWidget *widgetAction = widgetForAction(overAction);
 
             if (overAction != m_dropAction && overActionBK && widgetAction && m_dropAction)
             {
-                toolBar()->removeAction(m_dropAction);
+                removeAction(m_dropAction);
                 if (m_checkedAction)
                 {
                     m_checkedAction->setCheckable(false);
@@ -342,36 +336,36 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                 {
                     if ((dragEvent->pos().x() - widgetAction->pos().x()) > (widgetAction->width() / 2))
                     {
-                        if (toolBar()->actions().count() >  toolBar()->actions().indexOf(overAction) + 1)
+                        if (actions().count() >  actions().indexOf(overAction) + 1)
                         {
-                            toolBar()->insertAction(toolBar()->actions().at(toolBar()->actions().indexOf(overAction) + 1), m_dropAction);
+                            insertAction(actions().at(actions().indexOf(overAction) + 1), m_dropAction);
                         }
                         else
                         {
-                            toolBar()->addAction(m_dropAction);
+                            addAction(m_dropAction);
                         }
                     }
                     else
                     {
-                        toolBar()->insertAction(overAction, m_dropAction);
+                        insertAction(overAction, m_dropAction);
                     }
                 }
                 else
                 {
                     if ((dragEvent->pos().x() - widgetAction->pos().x()) >= (widgetAction->width() * 0.75))
                     {
-                        if (toolBar()->actions().count() >  toolBar()->actions().indexOf(overAction) + 1)
+                        if (actions().count() >  actions().indexOf(overAction) + 1)
                         {
-                            toolBar()->insertAction(toolBar()->actions().at(toolBar()->actions().indexOf(overAction) + 1), m_dropAction);
+                            insertAction(actions().at(actions().indexOf(overAction) + 1), m_dropAction);
                         }
                         else
                         {
-                            toolBar()->addAction(m_dropAction);
+                            addAction(m_dropAction);
                         }
                     }
                     else if ((dragEvent->pos().x() - widgetAction->pos().x()) <= (widgetAction->width() * 0.25))
                     {
-                        toolBar()->insertAction(overAction, m_dropAction);
+                        insertAction(overAction, m_dropAction);
                     }
                     else
                     {
@@ -392,7 +386,7 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
     {
         QDropEvent *dropEvent = static_cast<QDropEvent*>(event);
         KBookmark bookmark;
-        KBookmarkGroup root = rApp->bookmarkManager()->manager()->toolbar();
+        KBookmarkGroup root = BookmarkManager::self()->manager()->toolbar();
 
         if (m_checkedAction)
         {
@@ -403,7 +397,7 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
         if (dropEvent->mimeData()->hasFormat(BookmarkManager::bookmark_mime_type()))
         {
             QByteArray addresses = dropEvent->mimeData()->data(BookmarkManager::bookmark_mime_type());
-            bookmark =  rApp->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
+            bookmark =  BookmarkManager::self()->findByAddress(QString::fromLatin1(addresses.data()));
             if (bookmark.isNull())
                 return false;
         }
@@ -411,8 +405,9 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
         {
             // DROP is URL
             QString url = dropEvent->mimeData()->urls().at(0).toString();
-            QString title = url.contains(rApp->mainWindow()->currentTab()->url().url())
-                            ? rApp->mainWindow()->currentTab()->view()->title()
+            WebWindow *w = qobject_cast<WebWindow *>(parent());
+            QString title = url.contains(w->url().url())
+                            ? w->title()
                             : url;
             bookmark = root.addBookmark(title, url);
         }
@@ -423,8 +418,9 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
             KUrl u(url);
             if (u.isValid())
             {
-                QString title = url.contains(rApp->mainWindow()->currentTab()->url().url())
-                                ? rApp->mainWindow()->currentTab()->view()->title()
+                WebWindow *w = qobject_cast<WebWindow *>(parent());
+                QString title = url.contains(w->url().url())
+                                ? w->title()
                                 : url;
                 bookmark = root.addBookmark(title, url);
             }
@@ -434,23 +430,23 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
             return false;
         }
 
-        QAction *destAction = toolBar()->actionAt(dropEvent->pos());
+        QAction *destAction = actionAt(dropEvent->pos());
         if (destAction && destAction == m_dropAction)
         {
-            if (toolBar()->actions().indexOf(m_dropAction) > 0)
+            if (actions().indexOf(m_dropAction) > 0)
             {
-                destAction = toolBar()->actions().at(toolBar()->actions().indexOf(m_dropAction) - 1);
+                destAction = actions().at(actions().indexOf(m_dropAction) - 1);
             }
             else
             {
-                destAction = toolBar()->actions().at(1);
+                destAction = actions().at(1);
             }
         }
 
         if (destAction)
         {
             KBookmarkActionInterface *destBookmarkAction = dynamic_cast<KBookmarkActionInterface *>(destAction);
-            QWidget *widgetAction = toolBar()->widgetForAction(destAction);
+            QWidget *widgetAction = widgetForAction(destAction);
 
             if (destBookmarkAction && !destBookmarkAction->bookmark().isNull() && widgetAction
                     && bookmark.address() != destBookmarkAction->bookmark().address())
@@ -485,19 +481,19 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                 }
 
 
-                rApp->bookmarkManager()->emitChanged();
+                BookmarkManager::self()->emitChanged();
             }
         }
         else
         {
             root.deleteBookmark(bookmark);
             bookmark = root.addBookmark(bookmark);
-            if (dropEvent->pos().x() < toolBar()->widgetForAction(toolBar()->actions().first())->pos().x())
+            if (dropEvent->pos().x() < widgetForAction(actions().first())->pos().x())
             {
                 root.moveBookmark(bookmark, KBookmark());
             }
 
-            rApp->bookmarkManager()->emitChanged();
+            BookmarkManager::self()->emitChanged();
         }
         dropEvent->accept();
     }
@@ -510,18 +506,18 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
     QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
 
     // These events need to be handled only for Bookmark actions and not the bar
-    if (watched != toolBar() && mouseEvent)
+    if (watched != this && mouseEvent)
     {
         switch (event->type())
         {
         case QEvent::MouseButtonPress: // drag handling
         {
-            QPoint pos = toolBar()->mapFromGlobal(QCursor::pos());
-            KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(toolBar()->actionAt(pos));
+            QPoint pos = mapFromGlobal(QCursor::pos());
+            KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(actionAt(pos));
 
             if (action && mouseEvent->button() != Qt::MidButton)
             {
-                m_dragAction = toolBar()->actionAt(pos);
+                m_dragAction = actionAt(pos);
                 m_startDragPos = pos;
 
                 // The menu is displayed only when the mouse button is released
@@ -533,7 +529,7 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
 
         case QEvent::MouseMove:
         {
-            int distance = (toolBar()->mapFromGlobal(QCursor::pos()) - m_startDragPos).manhattanLength();
+            int distance = (mapFromGlobal(QCursor::pos()) - m_startDragPos).manhattanLength();
             if (!m_currentMenu && distance >= QApplication::startDragDistance())
             {
                 startDrag();
@@ -543,21 +539,34 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
 
         case QEvent::MouseButtonRelease:
         {
-            QPoint destPos = toolBar()->mapFromGlobal(QCursor::pos());
+            QPoint destPos = mapFromGlobal(QCursor::pos());
             int distance = (destPos - m_startDragPos).manhattanLength();
-            KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(toolBar()->actionAt(destPos));
+            KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(actionAt(destPos));
 
-            if (action && action->bookmark().isGroup())
+            if (action)
             {
-                if (mouseEvent->button() == Qt::MidButton)
+                if (action->bookmark().isGroup())
                 {
-                    rApp->bookmarkManager()->owner()->openBookmarkFolder(action->bookmark());
+                    if (mouseEvent->button() == Qt::MidButton)
+                    {
+                        BookmarkManager::self()->owner()->loadBookmarkFolder(action->bookmark());
+                    }
+                    else if (distance < QApplication::startDragDistance())
+                    {
+                        KBookmarkActionMenu *menu = dynamic_cast<KBookmarkActionMenu *>(actionAt(m_startDragPos));
+                        QPoint actionPos = mapToGlobal(widgetForAction(menu)->pos());
+                        menu->menu()->popup(QPoint(actionPos.x(), actionPos.y() + widgetForAction(menu)->height()));
+                    }
                 }
-                else if (distance < QApplication::startDragDistance())
+                else
                 {
-                    KBookmarkActionMenu *menu = dynamic_cast<KBookmarkActionMenu *>(toolBar()->actionAt(m_startDragPos));
-                    QPoint actionPos = toolBar()->mapToGlobal(toolBar()->widgetForAction(menu)->pos());
-                    menu->menu()->popup(QPoint(actionPos.x(), actionPos.y() + toolBar()->widgetForAction(menu)->height()));
+                    if (!action->bookmark().isNull() && !action->bookmark().isSeparator())
+                    {
+                        if (mouseEvent->button() == Qt::MidButton)
+                        {
+                            BookmarkManager::self()->owner()->loadBookmarkInNewTab(action->bookmark());
+                        }
+                    }
                 }
             }
         }
@@ -572,14 +581,6 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
 }
 
 
-void BookmarkToolBar::actionHovered()
-{
-    KBookmarkActionInterface* action = dynamic_cast<KBookmarkActionInterface *>(sender());
-    if (action)
-        rApp->mainWindow()->notifyMessage(action->bookmark().url().url());
-}
-
-
 void BookmarkToolBar::startDrag()
 {
     KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(m_dragAction);
@@ -592,7 +593,7 @@ void BookmarkToolBar::startDrag()
         mimeData->setData(BookmarkManager::bookmark_mime_type(), address);
         bookmark.populateMimeData(mimeData);
 
-        QDrag *drag = new QDrag(toolBar());
+        QDrag *drag = new QDrag(this);
         drag->setMimeData(mimeData);
 
         if (bookmark.isGroup())
@@ -601,7 +602,7 @@ void BookmarkToolBar::startDrag()
         }
         else
         {
-            drag->setPixmap(rApp->iconManager()->iconForUrl(action->bookmark().url()).pixmap(24, 24));
+            drag->setPixmap(IconManager::self()->iconForUrl(action->bookmark().url()).pixmap(24, 24));
         }
 
         drag->start(Qt::MoveAction);
