@@ -59,9 +59,6 @@ protected:
     virtual void refill();
     virtual QAction* actionForBookmark(const KBookmark &bookmark);
 
-private Q_SLOTS:
-    void actionHovered();
-
 private:
     void addOpenFolderInTabs();
 
@@ -80,21 +77,19 @@ private:
  * Some events from the toolbar are handled to allow the drag and drop
  */
 
-class BookmarkToolBar : public QObject
+class BookmarkToolBar : public KToolBar
 {
     Q_OBJECT
 
 public:
-    BookmarkToolBar(KToolBar *toolBar, QObject *parent);
-
-    KToolBar* toolBar();
+    explicit BookmarkToolBar(QWidget *parent);
+    ~BookmarkToolBar();
 
 protected:
     bool eventFilter(QObject *watched, QEvent *event);
 
 private Q_SLOTS:
     void contextMenu(const QPoint &);
-    void actionHovered();
     void menuDisplayed();
     void menuHidden();
     void hideMenu();
@@ -103,7 +98,6 @@ private Q_SLOTS:
 private:
     void startDrag();
 
-    KToolBar *m_toolBar;
     KMenu *m_currentMenu;
     QPoint m_startDragPos;
     QAction *m_dragAction;
