@@ -36,7 +36,6 @@
 // Local Includes
 #include "historymodels.h"
 #include "autosaver.h"
-#include "application.h"
 
 // KDE Includes
 #include <KStandardDirs>
@@ -44,6 +43,7 @@
 #include <KCompletion>
 
 // Qt Includes
+#include <QApplication>
 #include <QList>
 #include <QUrl>
 #include <QDate>
@@ -64,6 +64,22 @@
 static const unsigned int HISTORY_VERSION = 25;
 
 
+QWeakPointer<HistoryManager> HistoryManager::s_historyManager;
+
+
+HistoryManager *HistoryManager::self()
+{
+    if (s_historyManager.isNull())
+    {
+        s_historyManager = new HistoryManager(qApp);
+    }
+    return s_historyManager.data();
+}
+
+
+// ----------------------------------------------------------------------------------------------
+
+
 HistoryManager::HistoryManager(QObject *parent)
     : QObject(parent)
     , m_saveTimer(new AutoSaver(this))
@@ -91,6 +107,8 @@ HistoryManager::~HistoryManager()
         return;
     }
     m_saveTimer->saveIfNeccessary();
+
+    kDebug() << "bye bye history...";
 }
 
 
@@ -226,6 +244,18 @@ void HistoryManager::removeHistoryEntry(const KUrl &url, const QString &title)
 }
 
 
+void HistoryManager::removeHistoryLocationEntry(int value)
+{
+    if (value < 0)
+        return;
+    
+    HistoryItem item = m_history.at(value);
+    m_lastSavedUrl.clear();
+    m_history.removeOne(item);
+    emit entryRemoved(item);
+}
+
+
 QList<HistoryItem> HistoryManager::find(const QString &text)
 {
     QList<HistoryItem> list;
