@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2012 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -25,18 +25,24 @@
 
 
 // version include
-#include "config-version.h"
+#include <config-version.h>
 
 // Local Includes
 #include "application.h"
-#include "mainwindow.h"
+#include "sessionmanager.h"
+#include "tabwindow.h"
+#include "urlresolver.h"
 
 // KDE Includes
-#include <KAboutData>
-#include <KCmdLineArgs>
+#include <KDE/KAboutData>
+#include <KDE/KUniqueApplication>
+#include <KDE/KCmdLineArgs>
+#include <KDebug>
 
 // Qt Includes
 #include <QDir>
+#include <QUrl>
+
 
 static const char description[] =
     I18N_NOOP("A lightweight Web Browser for KDE based on WebKit");
@@ -55,6 +61,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                      "http://rekonq.kde.org"
                     );
 
+
     // --------------- about authors -----------------------------
     about.addAuthor(ki18n("Andrea Diamantini"),
                     ki18n("Project Lead, Developer, Maintainer"),
@@ -106,6 +113,12 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "tirtha.p.chatterjee@gmail.com",
                     "");
 
+    about.addAuthor(ki18n("Lindsay Mathieson"),
+                    ki18n("Implemented inline spellcheck, provided hints, discovered bugs"),
+                    "lindsay.mathieson@gmail.com",
+                    "");
+
+
     // --------------- about credits -----------------------------
     about.addCredit(ki18n("Dawit Alemayehu"),
                     ki18n("KDEWebKit (main) developer. And KIO. And KUriFilter. And more.."),
@@ -117,11 +130,6 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "adaptee@gmail.com",
                     "");
 
-    about.addCredit(ki18n("Lindsay Mathieson"),
-                    ki18n("Provided Patches & Hints, Discovered Bugs"),
-                    "lindsay.mathieson@gmail.com",
-                    "");
-
     about.addCredit(ki18n("Panagiotis Papadopoulos"),
                     ki18n("Quite everything but code"),
                     "pano_90@gmx.net",
@@ -182,21 +190,21 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "pino@kde.org",
                     "");
 
+
     // Initialize command line args
     KCmdLineArgs::init(argc, argv, &about);
 
     // Define the command line options using KCmdLineOptions
     KCmdLineOptions options;
 
-    // adding URL option
+    // adding options
+    options.add("incognito" , ki18n("Open in incognito mode"));
+    options.add("webapp" , ki18n("Open URL as web app (in a simple window)"));
     options.add("+[URL]" , ki18n("Location to open"));
 
     // Register the supported options
     KCmdLineArgs::addCmdLineOptions(options);
 
-    // Add options from Application class
-    Application::addCmdLineOptions();
-
     if (!Application::start())
     {
         kWarning() << "rekonq is already running!";
@@ -205,20 +213,24 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
 
 #if defined(Q_WS_X11)
     // On X11, the raster engine gives better performance than native.
-    QApplication::setGraphicsSystem(QL1S("raster"));
+    QApplication::setGraphicsSystem(QLatin1String("raster"));
 #endif
 
+    KCmdLineArgs::setCwd(QDir::currentPath().toUtf8());
+
     Application app;
 
     // set application data
-    QCoreApplication::setApplicationName(QL1S("rekonq"));
+    QCoreApplication::setApplicationName(QLatin1String("rekonq"));
     QCoreApplication::setApplicationVersion(REKONQ_VERSION);
 
-    KCmdLineArgs::setCwd(QDir::currentPath().toUtf8());
-
     if (app.isSessionRestored())
-        for (int i = 1; MainWindow::canBeRestored(i); i++)
-            app.newMainWindow(false)->restore(i);
+        for (int i = 1; TabWindow::canBeRestored(i); i++)
+        {
+            TabWindow * newTabWindow = app.newTabWindow(false);
+            if (newTabWindow->restore(i))
+                SessionManager::self()->restoreTabWindow(newTabWindow);
+        }
 
     return app.exec();
 }
