@@ -32,16 +32,22 @@
 
 // Local Includes
 #include "application.h"
-#include "historymanager.h"
-#include "mainview.h"
-#include "mainwindow.h"
-#include "webtab.h"
+#include "autosaver.h"
+#include "tabhistory.h"
+
+#include "tabwindow.h"
+#include "tabbar.h"
+
+#include "webwindow.h"
+#include "webpage.h"
 
 // KDE Includes
 #include <KStandardDirs>
+#include <KUrl>
 
 // Qt Includes
 #include <QFile>
+#include <QDomDocument>
 
 
 // Only used internally
@@ -68,50 +74,109 @@ bool readSessionDocument(QDomDocument & document, const QString & sessionFilePat
 }
 
 
-int loadViewTabs(MainView *mv, QDomElement & window, bool checkViewExists)
+int loadTabs(TabWindow *tw, QDomElement & window, bool useFirstTab, bool justThePinnedOnes = false)
 {
     int currentTab = 0;
 
     for (unsigned int tabNo = 0; tabNo < window.elementsByTagName("tab").length(); tabNo++)
     {
         QDomElement tab = window.elementsByTagName("tab").at(tabNo).toElement();
-        if (tab.hasAttribute("currentTab"))
-            currentTab = tabNo;
+        bool tabIsPinned = tab.hasAttribute("pinned");
+        kDebug() << "Tab #" << tabNo <<  " is pinned? " << tabIsPinned;
 
-        WebView * view = 0;
-        if (tabNo == 0 && checkViewExists)
-            view = mv->webTab(0)->view();
-        else
-            view = mv->newWebTab()->view();
+        if (!justThePinnedOnes || tabIsPinned)
+        {
+            if (tab.hasAttribute("currentTab"))
+                currentTab = tabNo;
 
-        QDomCDATASection historySection = tab.firstChild().toCDATASection();
-        QByteArray history = QByteArray::fromBase64(historySection.data().toAscii());
+            KUrl u = KUrl(tab.attribute("url"));
 
-        QDataStream readingStream(&history, QIODevice::ReadOnly);
-        readingStream >> *(view->history());
+            TabHistory tabHistory;
+            tabHistory.title = tab.attribute("title");
+            tabHistory.url = tab.attribute("url");
+            QDomCDATASection historySection = tab.firstChild().toCDATASection();
+            tabHistory.history = QByteArray::fromBase64(historySection.data().toAscii());
 
-        // Get sure about urls and/or pdf are loaded
-        KUrl u = KUrl(tab.attribute("url"));
-        view->load(u);
+            if (tabNo == 0 && useFirstTab)
+            {
+                tw->loadUrl(u, Rekonq::CurrentTab, &tabHistory);
+            }
+            else
+            {
+                tw->loadUrl(u, Rekonq::NewTab, &tabHistory);
+            }
+
+            if (tabIsPinned)
+            {
+                tw->tabBar()->setTabData(tabNo, true);
+                if (tw->tabBar()->tabButton(tabNo, QTabBar::RightSide))
+                    tw->tabBar()->tabButton(tabNo, QTabBar::RightSide)->hide(); // NOTE: this is not good here: where is its proper place?
+            }
+        }
     }
 
     return currentTab;
 }
 
 
-// ---------------------------------------------------------------------
+bool areTherePinnedTabs(QDomElement & window)
+{
+    bool b = false;
+
+    for (unsigned int tabNo = 0; tabNo < window.elementsByTagName("tab").length(); tabNo++)
+    {
+        QDomElement tab = window.elementsByTagName("tab").at(tabNo).toElement();
+        b = tab.hasAttribute("pinned");
+        if (b)
+            return true;
+    }
+
+    return b;
+}
+
+
+// -------------------------------------------------------------------------------------------------
+
+
+QWeakPointer<SessionManager> SessionManager::s_sessionManager;
+
+
+SessionManager *SessionManager::self()
+{
+    if (s_sessionManager.isNull())
+    {
+        s_sessionManager = new SessionManager(qApp);
+    }
+    return s_sessionManager.data();
+}
+
+
+// ----------------------------------------------------------------------------------------------
 
 
 SessionManager::SessionManager(QObject *parent)
     : QObject(parent)
     , m_safe(true)
     , m_isSessionEnabled(false)
+    , m_saveTimer(new AutoSaver(this))
 {
+    // AutoSaver. Save your hd from frying...
+    connect(m_saveTimer, SIGNAL(saveNeeded()), this, SLOT(save()));
+
     m_sessionFilePath = KStandardDirs::locateLocal("appdata" , "session");
 }
 
 
 void SessionManager::saveSession()
+{
+    if (!m_isSessionEnabled)
+        return;
+
+    m_saveTimer->changeOccurred();
+}
+
+
+void SessionManager::save()
 {
     if (!m_isSessionEnabled || !m_safe)
         return;
@@ -126,35 +191,38 @@ void SessionManager::saveSession()
         kDebug() << "Unable to open session file" << sessionFile.fileName();
         return;
     }
-    MainWindowList wl = rApp->mainWindowList();
+    TabWindowList wl = rApp->tabWindowList();
     QDomDocument document("session");
     QDomElement session = document.createElement("session");
     document.appendChild(session);
 
-    Q_FOREACH(const QWeakPointer<MainWindow> &w, wl)
+    Q_FOREACH(const QWeakPointer<TabWindow> &w, wl)
     {
-        MainView *mv = w.data()->mainView();
         QDomElement window = document.createElement("window");
         int tabInserted = 0;
 
         window.setAttribute("name", w.data()->objectName());
 
-        for (signed int tabNo = 0; tabNo < mv->count(); tabNo++)
+        for (signed int tabNo = 0; tabNo < w.data()->count(); tabNo++)
         {
-            KUrl u = mv->webTab(tabNo)->url();
+            KUrl u = w.data()->webWindow(tabNo)->url();
 
             tabInserted++;
             QDomElement tab = document.createElement("tab");
-            tab.setAttribute("title", mv->webTab(tabNo)->view()->title()); // redundant, but needed for closedSites()
+            tab.setAttribute("title", w.data()->webWindow(tabNo)->title()); // redundant, but needed for closedSites()
             // as there's not way to read out the historyData
             tab.setAttribute("url", u.url());
-            if (mv->currentIndex() == tabNo)
+            if (w.data()->currentIndex() == tabNo)
             {
                 tab.setAttribute("currentTab", 1);
             }
+            if (w.data()->tabBar()->tabData(tabNo).toBool()) // pinned tab info
+            {
+                tab.setAttribute("pinned", 1);
+            }
             QByteArray history;
             QDataStream historyStream(&history, QIODevice::ReadWrite);
-            historyStream << *(mv->webTab(tabNo)->view()->history());
+            historyStream << *(w.data()->webWindow(tabNo)->page()->history());
             QDomCDATASection historySection = document.createCDATASection(history.toBase64());
 
             tab.appendChild(historySection);
@@ -184,17 +252,44 @@ bool SessionManager::restoreSessionFromScratch()
     {
         QDomElement window = document.elementsByTagName("window").at(winNo).toElement();
 
-        MainView *mv = rApp->newMainWindow(false)->mainView();
+        TabWindow *tw = rApp->newTabWindow();
 
-        int currentTab = loadViewTabs(mv, window, false);
+        int currentTab = loadTabs(tw, window, true, false);
 
-        mv->setCurrentIndex(currentTab);
+        tw->setCurrentIndex(currentTab);
     }
 
     return true;
 }
 
 
+bool SessionManager::restoreJustThePinnedTabs()
+{
+    QDomDocument document("session");
+
+    if (!readSessionDocument(document, m_sessionFilePath))
+        return false;
+
+    bool done = false;
+    for (unsigned int winNo = 0; winNo < document.elementsByTagName("window").length(); winNo++)
+    {
+        QDomElement window = document.elementsByTagName("window").at(winNo).toElement();
+
+        if (!areTherePinnedTabs(window))
+            continue;
+
+        done = true;
+        TabWindow *tw = rApp->newTabWindow(false);
+
+        int currentTab = loadTabs(tw, window, false, true);
+
+        tw->setCurrentIndex(currentTab);
+    }
+
+    return done;
+}
+
+
 void SessionManager::restoreCrashedSession()
 {
     QDomDocument document("session");
@@ -206,43 +301,22 @@ void SessionManager::restoreCrashedSession()
     {
         QDomElement window = document.elementsByTagName("window").at(winNo).toElement();
 
-        MainView *mv = (winNo == 0) ? rApp->mainWindow()->mainView() : rApp->newMainWindow()->mainView();
+        TabWindow *tw = (winNo == 0)
+                        ? rApp->tabWindow()
+                        : rApp->newTabWindow();
 
-        bool useCurrentTab = (!mv->currentWebTab()->url().isValid() || (mv->currentWebTab()->url().protocol() == QL1S("about")));
-        int currentTab = loadViewTabs(mv, window, useCurrentTab);
+        KUrl u = tw->currentWebWindow()->url();
+        bool useCurrentTab = (u.isEmpty() || u.protocol() == QL1S("about"));
+        int currentTab = loadTabs(tw, window, useCurrentTab);
 
-        mv->setCurrentIndex(currentTab);
+        tw->setCurrentIndex(currentTab);
     }
 
     setSessionManagementEnabled(true);
 }
 
 
-int SessionManager::restoreSavedSession()
-{
-    QDomDocument document("session");
-
-    if (!readSessionDocument(document, m_sessionFilePath))
-        return 0;
-
-    unsigned int winNo;
-
-    for (winNo = 0; winNo < document.elementsByTagName("window").length(); winNo++)
-    {
-        QDomElement window = document.elementsByTagName("window").at(winNo).toElement();
-
-        MainView *mv = rApp->newMainWindow()->mainView();
-
-        int currentTab = loadViewTabs(mv, window, true);
-
-        mv->setCurrentIndex(currentTab);
-    }
-
-    return winNo;
-}
-
-
-bool SessionManager::restoreMainWindow(MainWindow* window)
+bool SessionManager::restoreTabWindow(TabWindow* window)
 {
     QDomDocument document("session");
 
@@ -258,11 +332,9 @@ bool SessionManager::restoreMainWindow(MainWindow* window)
         if (window->objectName() != savedWindowElement.attribute("name", ""))
             continue;
 
-        MainView *mv = window->mainView();
+        int currentTab = loadTabs(window, savedWindowElement, false);
 
-        int currentTab = loadViewTabs(mv, savedWindowElement, false);
-
-        mv->setCurrentIndex(currentTab);
+        window->setCurrentIndex(currentTab);
 
         return true;
     }
