@@ -36,19 +36,29 @@
 // Qt Includes
 #include <QObject>
 #include <QString>
+#include <QWeakPointer>
 
 // Forward Declarations
+class AutoSaver;
 class TabHistory;
-class MainWindow;
+class TabWindow;
+
 
 /**
   * Session Management: Needs clean up :)
+  *
   */
 class REKONQ_TESTS_EXPORT SessionManager : public QObject
 {
     Q_OBJECT
+
 public:
-    SessionManager(QObject *parent = 0);
+    /**
+     * Entry point.
+     * Access to SessionManager class by using
+     * SessionManager::self()->thePublicMethodYouNeed()
+     */
+    static SessionManager *self();
 
     inline void setSessionManagementEnabled(bool on)
     {
@@ -57,21 +67,26 @@ public:
 
     QList<TabHistory> closedSites();
 
-    // This method restores session
-    // while turning back from Private mode
-    int restoreSavedSession();
+    // This method restores a single TabWindow
+    bool restoreTabWindow(TabWindow * window);
 
-    // This method restores a single MainWindow
-    bool restoreMainWindow(MainWindow * window);
+private:
+    explicit SessionManager(QObject *parent = 0);
 
 public Q_SLOTS:
     // This method restores session
     // on restart when restore at startup is chosen
     bool restoreSessionFromScratch();
 
-private Q_SLOTS:
+    // This method restores (eventually) the tabs present
+    // if there are NO pinned tabs to restore, it returns FALSE...
+    bool restoreJustThePinnedTabs();
+
     void saveSession();
 
+private Q_SLOTS:
+    void save();
+
     // This method restores session
     // after a crash
     void restoreCrashedSession();
@@ -81,6 +96,9 @@ private:
 
     bool m_safe;
     bool m_isSessionEnabled;
+    AutoSaver *m_saveTimer;
+
+    static QWeakPointer<SessionManager> s_sessionManager;
 };
 
 
