@@ -32,7 +32,7 @@
 #include "ui_settings_general.h"
 
 // Qt Includes
-#include <QtGui/QWidget>
+#include <QWidget>
 
 
 class GeneralWidget : public QWidget, private Ui::general
@@ -40,7 +40,7 @@ class GeneralWidget : public QWidget, private Ui::general
     Q_OBJECT
 
 public:
-    GeneralWidget(QWidget *parent = 0);
+    explicit GeneralWidget(QWidget *parent = 0);
 
     void save();
     bool changed();
