@@ -35,8 +35,6 @@
 #include "syncassistant.h"
 #include "syncmanager.h"
 
-#include "application.h"
-
 // KDE Includes
 #include <KStandardDirs>
 #include <KIcon>
@@ -93,9 +91,9 @@ void SyncCheckWidget::initializePage()
     QString loadingGitPath = KStandardDirs::locate("appdata" , "pics/loading.mng");
 
     // Now, load syncManager settings...
-    rApp->syncManager()->loadSettings();
+    SyncManager::self()->loadSettings();
 
-    SyncHandler *h = rApp->syncManager()->handler();
+    SyncHandler *h = SyncManager::self()->handler();
 
     if (!h)
     {
@@ -105,7 +103,7 @@ void SyncCheckWidget::initializePage()
         return;
     }
 
-    connect(h, SIGNAL(syncStatus(Rekonq::SyncData, bool, QString)), this, SLOT(updateWidget(Rekonq::SyncData, bool, QString)));
+    connect(h, SIGNAL(syncStatus(Rekonq::SyncData,bool,QString)), this, SLOT(updateWidget(Rekonq::SyncData,bool,QString)));
 
 
     // bookmarks
