@@ -44,7 +44,13 @@ class REKONQ_TESTS_EXPORT SyncManager : public QObject
     Q_OBJECT
 
 public:
-    SyncManager(QObject *parent = 0);
+    /**
+     * Entry point.
+     * Access to SyncManager class by using
+     * SyncManager::self()->thePublicMethodYouNeed()
+     */
+    static SyncManager *self();
+
     ~SyncManager();
 
     SyncHandler *handler() const
@@ -52,6 +58,9 @@ public:
         return _syncImplementation.data();
     };
 
+private:
+    SyncManager(QObject *parent = 0);
+
 public Q_SLOTS:
     void syncBookmarks();
     void syncHistory();
@@ -62,6 +71,8 @@ public Q_SLOTS:
 
 private:
     QWeakPointer<SyncHandler> _syncImplementation;
+
+    static QWeakPointer<SyncManager> s_syncManager;
 };
 
 #endif // SYNC_MANAGER_H
