@@ -26,12 +26,11 @@
 * ============================================================ */
 
 
-// Auto Includes
+// Self Includes
 #include "bookmarkwidget.h"
 #include "bookmarkwidget.moc"
 
 // Local includes
-#include "application.h"
 #include "bookmarkmanager.h"
 #include "bookmarkowner.h"
 
@@ -41,7 +40,6 @@
 #include <KIcon>
 #include <KLineEdit>
 #include <KRatingWidget>
-#include <KBookmarkDialog>
 
 // Qt Includes
 #include <QDialogButtonBox>
@@ -53,9 +51,6 @@
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 
-// Nepomuk config include
-#include "config-nepomuk.h"
-
 #ifdef HAVE_NEPOMUK
 // Local Nepomuk Includes
 #include "resourcelinkdialog.h"
@@ -68,13 +63,15 @@
 
 BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
     : QMenu(parent)
-    , m_bookmark(bookmark)
+    , m_bookmark(new KBookmark(bookmark))
+    , m_tagLine(new KLineEdit(this))
+    , m_commentEdit(new QPlainTextEdit(this))
 {
     setAttribute(Qt::WA_DeleteOnClose);
     setFixedWidth(320);
 
 #ifdef HAVE_NEPOMUK
-    m_nfoResource = (QUrl)m_bookmark.url();
+    m_nfoResource = (QUrl)m_bookmark->url();
     m_isNepomukEnabled = QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.NepomukServer");
     kDebug() << "IS NEPOMUK ACTUALLY RUNNING? " << m_isNepomukEnabled;
 #endif
@@ -107,19 +104,18 @@ BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
     m_folder = new KComboBox(this);
     layout->addRow(folderLabel, m_folder);
     setupFolderComboBox();
-    connect(m_folder, SIGNAL(currentIndexChanged(int)), this, SLOT(onFolderIndexChanged(int)));
 
     // Bookmark name
     QLabel *nameLabel = new QLabel(this);
     nameLabel->setText(i18n("Name:"));
     m_name = new KLineEdit(this);
-    if (m_bookmark.isNull())
+    if (m_bookmark->isNull())
     {
         m_name->setEnabled(false);
     }
     else
     {
-        m_name->setText(m_bookmark.text());
+        m_name->setText(m_bookmark->text());
         m_name->setFocus();
     }
     layout->addRow(nameLabel, m_name);
@@ -143,7 +139,6 @@ BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
         QLabel *commentLabel = new QLabel(this);
         commentLabel->setText(i18n("Describe:"));
         commentLabel->setAlignment(Qt::AlignCenter);
-        m_commentEdit = new QPlainTextEdit(this);
         if (!m_nfoResource.description().isEmpty())
         {
             m_commentEdit->setPlainText(m_nfoResource.description());
@@ -155,12 +150,11 @@ BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
         QLabel *tagLabel = new QLabel(this);
         tagLabel->setText(i18n("Tags:"));
         tagLabel->setAlignment(Qt::AlignLeft);
-        m_tagLine = new KLineEdit(this);
         m_tagLine->setPlaceholderText(i18n("add tags(comma separated)"));
 
 
         QList<Nepomuk::Tag> tagList = Nepomuk::Tag::allTags();
-        Q_FOREACH(Nepomuk::Tag t, tagList)
+        Q_FOREACH(const Nepomuk::Tag &t, tagList)
         {
             m_tList.append(t.label());
         }
@@ -195,6 +189,12 @@ BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
 }
 
 
+BookmarkWidget::~BookmarkWidget()
+{
+    delete m_bookmark;
+}
+
+
 void BookmarkWidget::showAt(const QPoint &pos)
 {
     adjustSize();
@@ -207,18 +207,18 @@ void BookmarkWidget::showAt(const QPoint &pos)
 
 void BookmarkWidget::accept()
 {
-    if (!m_bookmark.isNull() && m_name->text() != m_bookmark.fullText())
+    if (!m_bookmark->isNull() && m_name->text() != m_bookmark->fullText())
     {
-        m_bookmark.setFullText(m_name->text());
-        rApp->bookmarkManager()->emitChanged();
+        m_bookmark->setFullText(m_name->text());
+        BookmarkManager::self()->emitChanged();
     }
     QString folderAddress = m_folder->itemData(m_folder->currentIndex()).toString();
-    KBookmarkGroup a = rApp->bookmarkManager()->manager()->findByAddress(folderAddress).toGroup();
+    KBookmarkGroup a = BookmarkManager::self()->manager()->findByAddress(folderAddress).toGroup();
 
-    KBookmarkGroup parent = m_bookmark.parentGroup();
-    parent.deleteBookmark(m_bookmark);
-    a.addBookmark(m_bookmark);
-    rApp->bookmarkManager()->manager()->emitChanged(a);
+    KBookmarkGroup parent = m_bookmark->parentGroup();
+    parent.deleteBookmark(*m_bookmark);
+    a.addBookmark(*m_bookmark);
+    BookmarkManager::self()->manager()->emitChanged(a);
 
 #ifdef HAVE_NEPOMUK
     if (m_isNepomukEnabled)
@@ -233,8 +233,8 @@ void BookmarkWidget::accept()
 
 void BookmarkWidget::setupFolderComboBox()
 {
-    KBookmarkGroup toolBarRoot = rApp->bookmarkManager()->manager()->toolbar();
-    KBookmarkGroup root = rApp->bookmarkManager()->rootGroup();
+    KBookmarkGroup toolBarRoot = BookmarkManager::self()->manager()->toolbar();
+    KBookmarkGroup root = BookmarkManager::self()->rootGroup();
 
     if (toolBarRoot.address() == root.address())
     {
@@ -250,56 +250,36 @@ void BookmarkWidget::setupFolderComboBox()
     }
     m_folder->insertSeparator(1);
 
-    if (m_bookmark.parentGroup().address() != toolBarRoot.address())
+    if (m_bookmark->parentGroup().address() != toolBarRoot.address())
     {
-        QString parentText = m_bookmark.parentGroup().text();
+        QString parentText = m_bookmark->parentGroup().text();
 
-        if (m_bookmark.parentGroup().address() == root.address())
+        if (m_bookmark->parentGroup().address() == root.address())
         {
             parentText = i18n("Root folder");
         }
 
         m_folder->addItem(parentText,
-                          m_bookmark.parentGroup().address());
+                          m_bookmark->parentGroup().address());
         m_folder->insertSeparator(3);
     }
 
     for (KBookmark bookmark = toolBarRoot.first(); !bookmark.isNull(); bookmark = toolBarRoot.next(bookmark))
     {
-        if (bookmark.isGroup() && bookmark.address() != m_bookmark.parentGroup().address())
+        if (bookmark.isGroup() && bookmark.address() != m_bookmark->parentGroup().address())
         {
             m_folder->addItem(bookmark.text(), bookmark.address());
         }
     }
 
-    m_folder->insertSeparator(m_folder->count());
-    m_folder->addItem(KIcon("folder"), i18n("Choose..."));
-
-    int index =  m_folder->findData(m_bookmark.parentGroup().address());
+    int index =  m_folder->findData(m_bookmark->parentGroup().address());
     m_folder->setCurrentIndex(index);
 }
 
 
-void BookmarkWidget::onFolderIndexChanged(int index)
-{
-    if (index == m_folder->count() - 1)
-    {
-        KBookmarkDialog dialog(rApp->bookmarkManager()->manager());
-        KBookmarkGroup selectedGroup = dialog.selectFolder(m_bookmark.parentGroup());
-
-        if (selectedGroup.address() != m_bookmark.parentGroup().address() && !selectedGroup.isNull())
-        {
-            m_bookmark.parentGroup().deleteBookmark(m_bookmark);
-            selectedGroup.addBookmark(m_bookmark);
-            rApp->bookmarkManager()->manager()->emitChanged();
-        }
-    }
-}
-
-
 void BookmarkWidget::removeBookmark()
 {
-    rApp->bookmarkManager()->owner()->deleteBookmark(m_bookmark);
+    BookmarkManager::self()->owner()->deleteBookmark(*m_bookmark);
     close();
 
     emit updateIcon();
@@ -316,6 +296,7 @@ void BookmarkWidget::addTags(QList<Nepomuk::Tag> tagList)
             m_nfoResource.addTag(tag);
         }
     }
+
     Q_FOREACH(Nepomuk::Tag tag, m_nfoResource.tags())
     {
         if (!tagList.contains(tag))
@@ -377,7 +358,9 @@ void BookmarkWidget::addCommentSlot()
 
 void BookmarkWidget::linkToResourceSlot()
 {
-    Nepomuk::ResourceLinkDialog r(m_nfoResource);
-    r.exec();
+    QPointer<Nepomuk::ResourceLinkDialog> r = new Nepomuk::ResourceLinkDialog(m_nfoResource);
+    r->exec();
+    
+    r->deleteLater();
 }
 #endif
