@@ -34,19 +34,14 @@
 #include <QGridLayout>
 #include <QPlainTextEdit>
 
-// KDE Includes
-#include <KBookmark>
-
 // Nepomuk config include
-#include "config-nepomuk.h"
-
-#ifdef HAVE_NEPOMUK
+#include <config-nepomuk.h>
 
 // Nepomuk Includes
+#ifdef HAVE_NEPOMUK
 #include <Nepomuk/Resource>
 #include <Nepomuk/Tag>
 #include <Nepomuk/Vocabulary/NFO>
-
 #endif
 
 // Forward Declarations
@@ -61,6 +56,7 @@ class BookmarkWidget : public QMenu
 
 public:
     explicit BookmarkWidget(const KBookmark &bookmark, QWidget *parent = 0);
+    virtual ~BookmarkWidget();
 
     void showAt(const QPoint &pos);
 
@@ -79,7 +75,6 @@ private:
 private Q_SLOTS:
     void accept();
     void removeBookmark();
-    void onFolderIndexChanged(int index);
 
 #ifdef HAVE_NEPOMUK
     void setRatingSlot(int rate);
@@ -88,7 +83,7 @@ private Q_SLOTS:
 #endif
 
 private:
-    KBookmark m_bookmark;
+    KBookmark *m_bookmark;
     KLineEdit *m_name;
     KComboBox *m_folder;
     KLineEdit *m_tagLine;
