@@ -32,9 +32,10 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "application.h"
-#include "listitem.h"
 #include "searchengine.h"
+#include "urlresolver.h"
+
+#include "listitem.h"
 #include "urlbar.h"
 
 // KDE Includes
@@ -42,6 +43,7 @@
 #include <KUrl>
 
 // Qt Includes
+#include <QApplication>
 #include <QPoint>
 #include <QSize>
 #include <QEvent>
@@ -66,16 +68,16 @@ CompletionWidget::CompletionWidget(QWidget *parent)
 }
 
 
-void CompletionWidget::insertItems(const UrlSearchList &list, const QString& text, int offset)
+void CompletionWidget::insertItems(const UrlSuggestionList &list, const QString& text, int offset)
 {
-    Q_FOREACH(const UrlSearchItem & item, list)
+    Q_FOREACH(const UrlSuggestionItem & item, list)
     {
         ListItem *suggestion = ListItemFactory::create(item, text, this);
         suggestion->setBackgroundRole(offset % 2 ? QPalette::AlternateBase : QPalette::Base);
         connect(suggestion,
-                SIGNAL(itemClicked(ListItem*, Qt::MouseButton, Qt::KeyboardModifiers)),
+                SIGNAL(itemClicked(ListItem*,Qt::MouseButton,Qt::KeyboardModifiers)),
                 this,
-                SLOT(itemChosen(ListItem*, Qt::MouseButton, Qt::KeyboardModifiers)));
+                SLOT(itemChosen(ListItem*,Qt::MouseButton,Qt::KeyboardModifiers)));
         connect(suggestion, SIGNAL(updateList()), this, SLOT(updateList()));
         connect(this, SIGNAL(nextItemSubChoice()), suggestion, SLOT(nextItemSubChoice()));
 
@@ -85,7 +87,7 @@ void CompletionWidget::insertItems(const UrlSearchList &list, const QString& tex
 }
 
 
-void CompletionWidget::updateSearchList(const UrlSearchList &list, const QString& text)
+void CompletionWidget::updateSuggestionList(const UrlSuggestionList &list, const QString& text)
 {
     if (_hasSuggestions || _typedString != text)
         return;
@@ -98,7 +100,7 @@ void CompletionWidget::updateSearchList(const UrlSearchList &list, const QString
         insertItems(_resList, text);
         _list = _resList;
 
-        UrlSearchList sugList = list.mid(0, 4);
+        UrlSuggestionList sugList = list.mid(0, 4);
         insertItems(sugList, text, _list.count());
         _list.append(sugList);
         popup();
@@ -259,14 +261,8 @@ bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
 
             case Qt::Key_Enter:
             case Qt::Key_Return:
+            {
                 w = qobject_cast<UrlBar *>(parent());
-                if (kev->modifiers() == Qt::AltModifier)
-                {
-                    if (kev->key() == Qt::Key_Return || kev->key() == Qt::Key_Enter)
-                    {
-                        emit chosenUrl(w->text(), Rekonq::NewFocusedTab);
-                    }
-                }
 
                 if (!w->text().startsWith(QL1S("http://"), Qt::CaseInsensitive))
                 {
@@ -294,37 +290,48 @@ bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
                             url.setHost(host);
                         }
 
-                        emit chosenUrl(url, Rekonq::CurrentTab);
+                        if (url.isValid())
+                        {
+                            emit chosenUrl(url, Rekonq::CurrentTab);
+                            kev->accept();
+                            hide();
+                            return true;
+                        }
                     }
                 }
 
-                kDebug() << "Suggestion INDEX chosen: " << _currentIndex;
+                KUrl urlToLoad;
+                Rekonq::OpenType type = Rekonq::CurrentTab;
+
                 if (_currentIndex == -1)
                     _currentIndex = 0;
                 child = findChild<ListItem *>(QString::number(_currentIndex));
 
                 if (child) //the completionwidget is visible and the user had press down
                 {
-                    //we can use the url of the listitem
-                    emit chosenUrl(child->url(), Rekonq::CurrentTab);
+                    urlToLoad = child->url();
                 }
                 else //the user type too fast (completionwidget not visible or suggestion not downloaded)
                 {
-                    UrlResolver res(w->text());
-                    UrlSearchList list = res.orderedSearchItems();
-                    if (list.isEmpty())
+                    urlToLoad = UrlResolver::urlFromTextTyped(w->text());
+                }
+
+                if (kev->modifiers() == Qt::AltModifier)
+                {
+                    if (kev->key() == Qt::Key_Return || kev->key() == Qt::Key_Enter)
                     {
-                        emit chosenUrl(KUrl(_typedString), Rekonq::CurrentTab);
-                    }
-                    else
-                    {
-                        emit chosenUrl(list.first().url, Rekonq::CurrentTab);
+                        type = Rekonq::NewFocusedTab;
                     }
                 }
-                kev->accept();
-                hide();
-                return true;
 
+                hide();
+                emit chosenUrl(urlToLoad, type);
+                kev->accept();
+
+                if (type != Rekonq::CurrentTab)
+                    w->clear();
+                return true;
+            }
             case Qt::Key_Escape:
                 hide();
                 return true;
@@ -340,20 +347,20 @@ void CompletionWidget::setVisible(bool visible)
 {
     if (visible)
     {
-        rApp->installEventFilter(this);
+        qApp->installEventFilter(this);
     }
     else
     {
-        rApp->removeEventFilter(this);
+        qApp->removeEventFilter(this);
     }
 
-
     QFrame::setVisible(visible);
 }
 
 
 void CompletionWidget::itemChosen(ListItem *item, Qt::MouseButton button, Qt::KeyboardModifiers modifier)
 {
+    hide();
     if (button == Qt::MidButton
             || modifier == Qt::ControlModifier)
     {
@@ -363,7 +370,6 @@ void CompletionWidget::itemChosen(ListItem *item, Qt::MouseButton button, Qt::Ke
     {
         emit chosenUrl(item->url(), Rekonq::CurrentTab);
     }
-    hide();
 }
 
 
@@ -387,16 +393,26 @@ void CompletionWidget::suggestUrls(const QString &text)
         return;
     }
 
-    if (!isVisible())
-    {
-        UrlResolver::setSearchEngine(SearchEngine::defaultEngine());
-    }
-
-    UrlResolver *res = new UrlResolver(text);
-    connect(res, SIGNAL(suggestionsReady(UrlSearchList, QString)),
-            this, SLOT(updateSearchList(UrlSearchList, QString)));
+    UrlSuggester *res = new UrlSuggester(text);
+    connect(res, SIGNAL(suggestionsReady(UrlSuggestionList,QString)),
+            this, SLOT(updateSuggestionList(UrlSuggestionList,QString)));
     _resList = res->orderedSearchItems();
 
     // NOTE: It's important to call this AFTER orderedSearchItems() to let everything work
     res->computeSuggestions();
 }
+
+
+KUrl CompletionWidget::activeSuggestion()
+{
+    int index = _currentIndex;
+    if (_currentIndex == -1)
+        index = 0;
+
+    ListItem *child = findChild<ListItem *>(QString::number(index));
+    if (child)
+        return child->url();
+
+    kDebug() << "WARNING: NO URL to LOAD...";
+    return KUrl();
+}
