@@ -32,13 +32,14 @@
 #include "rekonq.h"
 
 // Local Includes
-#include "urlresolver.h"
-#include "application.h"
-#include "websnap.h"
-#include "completionwidget.h"
-#include "searchengine.h"
 #include "iconmanager.h"
 
+#include "urlsuggester.h"
+#include "completionwidget.h"
+
+#include "websnap.h"
+#include "searchengine.h"
+
 // KDE Includes
 #include <KIcon>
 #include <KAction>
@@ -46,6 +47,7 @@
 #include <kio/scheduler.h>
 
 // Qt Includes
+#include <QApplication>
 #include <QActionGroup>
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -59,7 +61,7 @@
 #include <QBitArray>
 
 
-ListItem::ListItem(const UrlSearchItem &item, QWidget *parent)
+ListItem::ListItem(const UrlSuggestionItem &item, QWidget *parent)
     : QWidget(parent)
     , m_option()
     , m_url(item.url)
@@ -69,7 +71,7 @@ ListItem::ListItem(const UrlSearchItem &item, QWidget *parent)
 
     // use the same application palette (hence, the same colors)
     // Qt docs says that using this cctor is possible & fast (qt:qpalette)
-    QPalette p(rApp->palette());
+    QPalette p(QApplication::palette());
     setPalette(p);
 
     deactivate();
@@ -132,7 +134,7 @@ void ListItem::leaveEvent(QEvent *e)
 void ListItem::mousePressEvent(QMouseEvent *e)
 {
     emit itemClicked(this, e->button(), e->modifiers());
-    QWidget::mousePressEvent(e);
+    e->accept();
 }
 
 
@@ -144,7 +146,7 @@ KUrl ListItem::url()
 
 QString ListItem::text()
 {
-    return m_url.url();
+    return m_url.prettyUrl();
 }
 
 
@@ -166,15 +168,15 @@ TypeIconLabel::TypeIconLabel(int type, QWidget *parent)
     hLayout->setAlignment(Qt::AlignRight);
     setLayout(hLayout);
 
-    if (type & UrlSearchItem::Search)
+    if (type & UrlSuggestionItem::Search)
         hLayout->addWidget(getIcon("edit-find"));
-    if (type & UrlSearchItem::Browse)
+    if (type & UrlSuggestionItem::Browse)
         hLayout->addWidget(getIcon("applications-internet"));
-    if (type & UrlSearchItem::Bookmark)
+    if (type & UrlSuggestionItem::Bookmark)
         hLayout->addWidget(getIcon("rating"));
-    if (type & UrlSearchItem::History)
+    if (type & UrlSuggestionItem::History)
         hLayout->addWidget(getIcon("view-history"));
-    if (type & UrlSearchItem::Suggestion)
+    if (type & UrlSuggestionItem::Suggestion)
         hLayout->addWidget(getIcon("help-hint"));
 }
 
@@ -195,7 +197,7 @@ QLabel *TypeIconLabel::getIcon(QString icon)
 IconLabel::IconLabel(const QString &icon, QWidget *parent)
     : QLabel(parent)
 {
-    QPixmap pixmapIcon = rApp->iconManager()->iconForUrl(KUrl(icon)).pixmap(16);
+    QPixmap pixmapIcon = IconManager::self()->iconForUrl(KUrl(icon)).pixmap(16);
     setFixedSize(16, 16);
     setPixmap(pixmapIcon);
 }
@@ -318,7 +320,7 @@ DescriptionLabel::DescriptionLabel(const QString &text, QWidget *parent)
 //--------------------------------------------------------------------------------------------
 
 
-PreviewListItem::PreviewListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+PreviewListItem::PreviewListItem(const UrlSuggestionItem &item, const QString &text, QWidget *parent)
     : ListItem(item, parent)
 {
     QHBoxLayout *hLayout = new QHBoxLayout;
@@ -391,8 +393,8 @@ ImageLabel::ImageLabel(const QString &url, int width, int height, QWidget *paren
     else
     {
         KIO::TransferJob *job = KIO::get(KUrl(url), KIO::NoReload, KIO::HideProgressInfo);
-        connect(job,  SIGNAL(data(KIO::Job*, QByteArray)),
-                this, SLOT(slotData(KJob*, QByteArray)));
+        connect(job,  SIGNAL(data(KIO::Job*,QByteArray)),
+                this, SLOT(slotData(KJob*,QByteArray)));
         connect(job,  SIGNAL(result(KJob*)),
                 this, SLOT(slotResult(KJob*)));
     }
@@ -419,14 +421,19 @@ void ImageLabel::slotResult(KJob *)
 // ---------------------------------------------------------------
 
 
-SearchListItem::SearchListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+SearchListItem::SearchListItem(const UrlSuggestionItem &item, const QString &text, QWidget *parent)
     : ListItem(item, parent)
     , m_text(text)
 {
-    m_iconLabel = new IconLabel(SearchEngine::buildQuery(UrlResolver::searchEngine(), ""), this);
+    m_iconLabel = new IconLabel(item.url, this);
     m_titleLabel = new TextLabel(this);
-    m_titleLabel->setEngineText(UrlResolver::searchEngine()->name(), item.title);
-    m_engineBar = new EngineBar(UrlResolver::searchEngine(), parent);
+    m_titleLabel->setEngineText(item.description, item.title);
+
+    KService::Ptr engine = SearchEngine::fromString(text);
+    if (!engine)
+        engine = SearchEngine::defaultEngine();
+
+    m_engineBar = new EngineBar(engine, parent);
 
     QHBoxLayout *hLayout = new QHBoxLayout;
     hLayout->setSpacing(4);
@@ -453,8 +460,15 @@ void SearchListItem::changeSearchEngine(KService::Ptr engine)
 {
     // NOTE: This to let rekonq loading text typed in the requested engine on click.
     // There probably is a better way to do it. I just cannot see it now...
-    UrlSearchItem item = UrlSearchItem(UrlSearchItem::Search, SearchEngine::buildQuery(engine, m_text), m_text);
-    SearchListItem sItem(item, m_text, this);
+
+    // remove the xx: part...
+    QString separator = SearchEngine::delimiter();
+    QString text = m_text.section(separator, 1, 1);
+
+    // create a new item && load it...
+    UrlSuggestionItem item = UrlSuggestionItem(UrlSuggestionItem::Search, SearchEngine::buildQuery(engine, text), text);
+
+    SearchListItem sItem(item, text, this);
     emit itemClicked(&sItem, Qt::LeftButton, Qt::NoModifier);
 }
 
@@ -479,6 +493,7 @@ EngineBar::EngineBar(KService::Ptr selectedEngine, QWidget *parent)
 
     if (SearchEngine::defaultEngine().isNull())
         return;
+
     m_engineGroup->addAction(newEngineAction(SearchEngine::defaultEngine(), selectedEngine));
     Q_FOREACH(const KService::Ptr & engine, SearchEngine::favorites())
     {
@@ -497,7 +512,7 @@ KAction *EngineBar::newEngineAction(KService::Ptr engine, KService::Ptr selected
     QUrl u = engine->property("Query").toUrl();
     KUrl url = KUrl(u.toString(QUrl::RemovePath | QUrl::RemoveQuery));
 
-    KAction *a = new KAction(rApp->iconManager()->iconForUrl(url), engine->name(), this);
+    KAction *a = new KAction(IconManager::self()->iconForUrl(url), engine->name(), this);
     a->setCheckable(true);
     if (engine->desktopEntryName() == selectedEngine->desktopEntryName()) a->setChecked(true);
     a->setData(engine->entryPath());
@@ -538,7 +553,7 @@ void EngineBar::selectNextEngine()
 // ---------------------------------------------------------------
 
 
-SuggestionListItem::SuggestionListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+SuggestionListItem::SuggestionListItem(const UrlSuggestionItem &item, const QString &text, QWidget *parent)
     : ListItem(item, parent)
     , m_text(item.title)
 {
@@ -562,7 +577,7 @@ QString SuggestionListItem::text()
 // ---------------------------------------------------------------
 
 
-VisualSuggestionListItem::VisualSuggestionListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+VisualSuggestionListItem::VisualSuggestionListItem(const UrlSuggestionItem &item, const QString &text, QWidget *parent)
     : ListItem(item, parent)
     , m_text(item.title)
 {
@@ -608,7 +623,7 @@ QString VisualSuggestionListItem::text()
 // ---------------------------------------------------------------
 
 
-BrowseListItem::BrowseListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+BrowseListItem::BrowseListItem(const UrlSuggestionItem &item, const QString &text, QWidget *parent)
     : ListItem(item, parent)
 {
     QHBoxLayout *hLayout = new QHBoxLayout;
@@ -625,29 +640,29 @@ BrowseListItem::BrowseListItem(const UrlSearchItem &item, const QString &text, Q
 // ---------------------------------------------------------------
 
 
-ListItem *ListItemFactory::create(const UrlSearchItem &item, const QString &text, QWidget *parent)
+ListItem *ListItemFactory::create(const UrlSuggestionItem &item, const QString &text, QWidget *parent)
 {
-    if (item.type & UrlSearchItem::Search)
+    if (item.type & UrlSuggestionItem::Search)
     {
         return new SearchListItem(item, text, parent);
     }
 
-    if (item.type & UrlSearchItem::Browse)
+    if (item.type & UrlSuggestionItem::Browse)
     {
         return new BrowseListItem(item, text, parent);
     }
 
-    if (item.type & UrlSearchItem::History)
+    if (item.type & UrlSuggestionItem::History)
     {
         return new PreviewListItem(item, text, parent);
     }
 
-    if (item.type & UrlSearchItem::Bookmark)
+    if (item.type & UrlSuggestionItem::Bookmark)
     {
         return new PreviewListItem(item, text, parent);
     }
 
-    if (item.type & UrlSearchItem::Suggestion)
+    if (item.type & UrlSuggestionItem::Suggestion)
     {
         if (item.description.isEmpty())
         {
