@@ -30,9 +30,8 @@
 
 // Local includes
 #include "application.h"
+#include "tabwindow.h"
 #include "iconmanager.h"
-#include "mainwindow.h"
-#include "webtab.h"
 
 // KDE Includes
 #include <KComboBox>
@@ -76,7 +75,7 @@ RSSWidget::RSSWidget(const QMap< KUrl, QString > &map, QWidget *parent)
     m_agregators->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
     m_agregators->addItem(KIcon("akregator"), QString("Akregator"));
-    m_agregators->addItem(rApp->iconManager()->iconForUrl(KUrl("http://google.com/reader")), i18n("Google Reader"));
+    m_agregators->addItem(IconManager::self()->iconForUrl(KUrl("http://google.com/reader")), i18n("Google Reader"));
 
     layout->addRow(agregator, m_agregators);
 
@@ -133,7 +132,7 @@ void RSSWidget::accept()
 void RSSWidget::addWithGoogleReader(const QString &url)
 {
     KUrl toLoad = KUrl("http://www.google.com/ig/add?feedurl=" + url);
-    rApp->mainWindow()->currentTab()->view()->load(toLoad);
+    rApp->tabWindow()->loadUrl(toLoad);
 }
 
 
