@@ -34,18 +34,24 @@
 // Auto Includes
 #include "rekonq.h"
 
-// Local Includes
+// App Includes
 #include "application.h"
-#include "mainwindow.h"
+
+// Local Includes
+#include "adblockmanager.h"
+#include "bookmarkmanager.h"
+#include "iconmanager.h"
+
+#include "adblockwidget.h"
+#include "bookmarkwidget.h"
+#include "favoritewidget.h"
+#include "rsswidget.h"
+
+#include "completionwidget.h"
+#include "urlresolver.h"
+
 #include "webtab.h"
 #include "webpage.h"
-#include "webview.h"
-#include "completionwidget.h"
-#include "bookmarkmanager.h"
-#include "bookmarkowner.h"
-#include "bookmarkwidget.h"
-#include "iconmanager.h"
-#include "favoritewidget.h"
 #include "searchengine.h"
 #include "websnap.h"
 
@@ -54,7 +60,10 @@
 #include <KStandardDirs>
 #include <KColorScheme>
 #include <KMenu>
+#include <KIcon>
 #include <KIconLoader>
+#include <KStandardAction>
+#include <KAction>
 
 // Qt Includes
 #include <QPainter>
@@ -100,11 +109,13 @@ QString guessUrlWithCustomFirstLevel(const QString &str1, const QString &str2)
     return url.toString();
 }
 
+
 // -----------------------------------------------------------------------------------------------------------
 
 
 UrlBar::UrlBar(QWidget *parent)
     : KLineEdit(parent)
+    , _box(new CompletionWidget(this))
     , _tab(0)
     , _icon(new IconButton(this))
     , _suggestionTimer(new QTimer(this))
@@ -139,28 +150,26 @@ UrlBar::UrlBar(QWidget *parent)
 
     connect(_tab, SIGNAL(loadProgressing()), this, SLOT(update()));
 
-    connect(_tab->view(), SIGNAL(urlChanged(QUrl)), this, SLOT(setQUrl(QUrl)));
-    connect(_tab->view(), SIGNAL(loadFinished(bool)), this, SLOT(loadFinished()));
-    connect(_tab->view(), SIGNAL(loadStarted()), this, SLOT(clearRightIcons()));
-    connect(_tab->view(), SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
-
-    // search icon
-    connect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(QString)), this, SLOT(updateRightIcons()));
+    connect(_tab, SIGNAL(urlChanged(QUrl)), this, SLOT(setQUrl(QUrl)));
+    connect(_tab, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished()));
+    connect(_tab, SIGNAL(loadStarted()), this, SLOT(clearRightIcons()));
+    connect(_tab, SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
 
     // bookmark icon
-    connect(rApp->bookmarkManager(), SIGNAL(bookmarksUpdated()), this, SLOT(updateRightIcons()));
+    connect(BookmarkManager::self(), SIGNAL(bookmarksUpdated()), this, SLOT(updateRightIcons()));
+
+    // suggestions
+    connect(_box.data(), SIGNAL(chosenUrl(KUrl,Rekonq::OpenType)), this, SLOT(loadRequestedUrl(KUrl,Rekonq::OpenType)));
+    connect(this, SIGNAL(textEdited(QString)), this, SLOT(detectTypedString(QString)));
 
     _suggestionTimer->setSingleShot(true);
     connect(_suggestionTimer, SIGNAL(timeout()), this, SLOT(suggest()));
-
-    activateSuggestions(true);
 }
 
 
 UrlBar::~UrlBar()
 {
     _suggestionTimer->stop();
-    activateSuggestions(false);
     _box.clear();
 
     disconnect();
@@ -170,41 +179,39 @@ UrlBar::~UrlBar()
 void UrlBar::setQUrl(const QUrl& url)
 {
     if (url.scheme() == QL1S("about"))
-    {
-        clear();
-        setFocus();
-    }
-    else
-    {
-        clearFocus();
-        KLineEdit::setUrl(url);
-        setCursorPosition(0);
-        refreshFavicon();
-    }
+        return;
+
+    clearFocus();
+    KLineEdit::setUrl(url);
+    setCursorPosition(0);
+    refreshFavicon();
 }
 
 
 void UrlBar::loadRequestedUrl(const KUrl& url, Rekonq::OpenType type)
 {
-    activateSuggestions(false);
     clearFocus();
     setUrl(url);
     rApp->loadUrl(url, type);
 }
 
 
-void UrlBar::loadDigitedUrl()
+void UrlBar::loadTypedUrl()
 {
-    UrlResolver res(text());
-    UrlSearchList list = res.orderedSearchItems();
-    if (list.isEmpty())
+    KUrl urlToLoad;
+    if (!_box.isNull())
     {
-        loadRequestedUrl(KUrl(text()));
-    }
-    else
-    {
-        loadRequestedUrl(list.first().url);
+        urlToLoad = _box.data()->activeSuggestion();
+        if (!urlToLoad.isEmpty())
+        {
+            loadRequestedUrl(urlToLoad);
+            return;
+        }
     }
+
+    // fallback here
+    urlToLoad = UrlResolver::urlFromTextTyped(text());
+    loadRequestedUrl(urlToLoad);
 }
 
 
@@ -214,7 +221,7 @@ void UrlBar::paintEvent(QPaintEvent *event)
     QColor backgroundColor;
     QColor foregroundColor;
 
-    if (QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
+    if (_tab->page()->settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
     {
         backgroundColor = QColor(220, 220, 220);  // light gray
         foregroundColor = Qt::black;
@@ -324,7 +331,7 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
     {
         clearFocus();
         if (!(_tab->url().protocol() == QL1S("about")))
-            setText(_tab->url().url());
+            setText(_tab->url().prettyUrl());
         event->accept();
     }
 
@@ -334,9 +341,7 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
 
 void UrlBar::focusInEvent(QFocusEvent *event)
 {
-    activateSuggestions(true);
-    rApp->mainWindow()->updateTabActions();
-
+    emit focusIn();
     KLineEdit::focusInEvent(event);
 }
 
@@ -389,7 +394,7 @@ void UrlBar::loadFinished()
     if (_tab->hasRSSInfo())
     {
         IconButton *bt = addRightIcon(UrlBar::RSS);
-        connect(bt, SIGNAL(clicked(QPoint)), _tab, SLOT(showRSSInfo(QPoint)));
+        connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(showRSSInfo(QPoint)));
     }
 
     // show SSL
@@ -400,18 +405,11 @@ void UrlBar::loadFinished()
         connect(bt, SIGNAL(clicked(QPoint)), _tab->page(), SLOT(showSSLInfo(QPoint)));
     }
 
-    // show add search engine
-    if (_tab->hasNewSearchEngine())
-    {
-        IconButton *bt = addRightIcon(UrlBar::SearchEngine);
-        connect(bt, SIGNAL(clicked(QPoint)), _tab, SLOT(showSearchEngine(QPoint)));
-    }
-
-    if (_tab->hasAdBlockedElements())
+    // Show adblock
+    if (AdBlockManager::self()->isEnabled())
     {
         IconButton *bt = addRightIcon(UrlBar::AdBlock);
-
-        connect(bt, SIGNAL(clicked(QPoint)), (QObject *) rApp->adblockManager(), SLOT(showBlockedItemDialog()));
+        connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(manageAdBlock(QPoint)));
     }
 
     // we need to update urlbar after the right icon settings
@@ -433,30 +431,6 @@ void UrlBar::updateRightIcons()
 }
 
 
-void UrlBar::activateSuggestions(bool b)
-{
-    if (b)
-    {
-        if (_box.isNull())
-        {
-            _box = new CompletionWidget(this);
-            installEventFilter(_box.data());
-            connect(_box.data(), SIGNAL(chosenUrl(KUrl, Rekonq::OpenType)), this, SLOT(loadRequestedUrl(KUrl, Rekonq::OpenType)));
-
-            // activate suggestions on edit text
-            connect(this, SIGNAL(textChanged(QString)), this, SLOT(detectTypedString(QString)));
-        }
-    }
-    else
-    {
-        disconnect(this, SIGNAL(textChanged(QString)), this, SLOT(detectTypedString(QString)));
-        removeEventFilter(_box.data());
-        if (!_box.isNull())
-            _box.data()->deleteLater();
-    }
-}
-
-
 void UrlBar::mouseDoubleClickEvent(QMouseEvent *event)
 {
     Q_UNUSED(event);
@@ -471,17 +445,17 @@ void UrlBar::contextMenuEvent(QContextMenuEvent* event)
     const bool clipboardFilled = !rApp->clipboard()->text().isEmpty();
 
     // Cut
-    KAction *a = KStandardAction::cut(this, SLOT(cut()), this);
+    KAction *a = KStandardAction::cut(this, SLOT(cut()), &menu);
     a->setEnabled(hasSelectedText());
     menu.addAction(a);
 
     // Copy
-    a = KStandardAction::copy(this, SLOT(copy()), this);
+    a = KStandardAction::copy(this, SLOT(copy()), &menu);
     a->setEnabled(hasSelectedText());
     menu.addAction(a);
 
     // Paste
-    a = KStandardAction::paste(this, SLOT(paste()), this);
+    a = KStandardAction::paste(this, SLOT(paste()), &menu);
     a->setEnabled(clipboardFilled);
     menu.addAction(a);
 
@@ -489,19 +463,19 @@ void UrlBar::contextMenuEvent(QContextMenuEvent* event)
     const QString clipboardText = rApp->clipboard()->text();
     if (isValidURL(clipboardText) || clipboardText.isEmpty())
     {
-        a = new KAction(i18n("Paste && Go"), this);
+        a = new KAction(i18n("Paste && Go"), &menu);
         connect(a, SIGNAL(triggered(bool)), this, SLOT(pasteAndGo()));
     }
     else
     {
-        a = new KAction(i18n("Paste && Search"), this);
+        a = new KAction(i18n("Paste && Search"), &menu);
         connect(a, SIGNAL(triggered(bool)), this, SLOT(pasteAndSearch()));
     }
     a->setEnabled(clipboardFilled);
     menu.addAction(a);
 
     // Delete
-    a = new KAction(KIcon("edit-delete"), i18n("Delete"), this);
+    a = new KAction(KIcon("edit-delete"), i18n("Delete"), &menu);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(delSlot()));
     a->setEnabled(hasSelectedText());
     menu.addAction(a);
@@ -509,7 +483,7 @@ void UrlBar::contextMenuEvent(QContextMenuEvent* event)
     menu.addSeparator();
 
     // Select All
-    a = KStandardAction::selectAll(this, SLOT(selectAll()), this);
+    a = KStandardAction::selectAll(this, SLOT(selectAll()), &menu);
     a->setEnabled(!text().isEmpty());
     menu.addAction(a);
 
@@ -559,7 +533,7 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setToolTip(i18n("Show SSL Info"));
         break;
     case UrlBar::BK:
-        if (rApp->bookmarkManager()->bookmarkForUrl(_tab->url()).isNull())
+        if (BookmarkManager::self()->bookmarkForUrl(_tab->url()).isNull())
         {
             rightIcon->setIcon(KIcon("bookmarks").pixmap(32, 32, QIcon::Disabled));
             rightIcon->setToolTip(i18n("Bookmark this page"));
@@ -594,9 +568,20 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         }
         break;
     case UrlBar::AdBlock:
-        rightIcon->setIcon(KIcon("preferences-web-browser-adblock"));
-        rightIcon->setToolTip(i18n("There are elements blocked by AdBlock"));
+    {
+        QStringList hosts = ReKonfig::whiteReferer();
+        if (!hosts.contains(_tab->url().host()))
+        {
+            rightIcon->setIcon(KIcon("preferences-web-browser-adblock"));
+            rightIcon->setToolTip(i18n("AdBlock is enabled on this site"));
+        }
+        else
+        {
+            rightIcon->setIcon(KIcon("preferences-web-browser-adblock").pixmap(32, 32, QIcon::Disabled));
+            rightIcon->setToolTip(i18n("AdBlock is NOT enabled on this site"));
+        }
         break;
+    }
     default:
         ASSERT_NOT_REACHED("ERROR.. default non extant case!!");
         break;
@@ -648,20 +633,22 @@ void UrlBar::detectTypedString(const QString &typed)
 
     if (_suggestionTimer->isActive())
         _suggestionTimer->stop();
-    _suggestionTimer->start(50);
+    _suggestionTimer->start(100);
 }
 
 
 void UrlBar::suggest()
 {
     if (!_box.isNull())
+    {
         _box.data()->suggestUrls(text());
+    }
 }
 
 
 void UrlBar::refreshFavicon()
 {
-    if (QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
+    if (_tab->page()->settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
     {
         _icon->setIcon(KIcon("view-media-artist"));
         return;
@@ -673,7 +660,8 @@ void UrlBar::refreshFavicon()
         _icon->setIcon(KIcon("arrow-right"));
         return;
     }
-    _icon->setIcon(rApp->iconManager()->iconForUrl(u));
+
+    _icon->setIcon(IconManager::self()->iconForUrl(u));
 }
 
 
@@ -687,7 +675,7 @@ void UrlBar::pasteAndSearch()
 {
     KService::Ptr defaultEngine = SearchEngine::defaultEngine();
     if (defaultEngine)
-        loadRequestedUrl(KUrl(SearchEngine::buildQuery(defaultEngine, rApp->clipboard()->text())));
+        loadRequestedUrl(KUrl(SearchEngine::buildQuery(defaultEngine, QApplication::clipboard()->text())));
 }
 
 
@@ -702,11 +690,11 @@ void UrlBar::manageBookmarks()
     if (_tab->url().scheme() == QL1S("about"))
         return;
 
-    KBookmark bookmark = rApp->bookmarkManager()->bookmarkForUrl(_tab->url());
+    KBookmark bookmark = BookmarkManager::self()->bookmarkForUrl(_tab->url());
 
     if (bookmark.isNull())
     {
-        bookmark = rApp->bookmarkManager()->owner()->bookmarkCurrentPage();
+        bookmark = BookmarkManager::self()->bookmarkCurrentPage();
     }
 
     // calculate position
@@ -754,11 +742,27 @@ void UrlBar::manageFavorites(QPoint pos)
 
     // also, save a site snapshot
     WebSnap *snap = new WebSnap(_tab->url(), this);
+    Q_UNUSED(snap);
 
     updateRightIcons();
 }
 
 
+void UrlBar::manageAdBlock(QPoint pos)
+{
+    IconButton *bt = qobject_cast<IconButton *>(this->sender());
+    if (!bt)
+        return;
+
+    if (_tab->url().scheme() == QL1S("about"))
+        return;
+
+    AdBlockWidget *widget = new AdBlockWidget(_tab->url(), this);
+    connect(widget, SIGNAL(updateIcon()), this, SLOT(updateRightIcons()));
+    widget->showAt(pos);
+}
+
+
 void UrlBar::updateRightIconPosition(IconButton *icon, int iconsCount)
 {
     // NOTE: cannot show a (let's say) 16x16 icon in a 16x16 square.
@@ -771,3 +775,37 @@ void UrlBar::updateRightIconPosition(IconButton *icon, int iconsCount)
 
     icon->move(iconWidth, iconHeight);
 }
+
+
+void UrlBar::showRSSInfo(const QPoint &pos)
+{
+    QWebElementCollection col = _tab->page()->mainFrame()->findAllElements("link[type=\"application/rss+xml\"]");
+    col.append(_tab->page()->mainFrame()->findAllElements("link[type=\"application/atom+xml\"]"));
+
+    QMap<KUrl, QString> map;
+
+    Q_FOREACH(const QWebElement & el, col)
+    {
+        QString urlString;
+        if (el.attribute("href").startsWith(QL1S("http")))
+            urlString = el.attribute("href");
+        else
+        {
+            KUrl u = _tab->url();
+            // NOTE
+            // cd() is probably better than setPath() here,
+            // for all those url sites just having a path
+            if (u.cd(el.attribute("href")))
+                urlString = u.toMimeDataString();
+        }
+
+        QString title = el.attribute("title");
+        if (title.isEmpty())
+            title = el.attribute("href");
+
+        map.insert(KUrl(urlString), title);
+    }
+
+    RSSWidget *widget = new RSSWidget(map, window());
+    widget->showAt(pos);
+}
