@@ -53,7 +53,7 @@ class IconButton : public QToolButton
     Q_OBJECT
 
 public:
-    IconButton(QWidget *parent = 0);
+    explicit IconButton(QWidget *parent = 0);
 
 Q_SIGNALS:
     void clicked(QPoint);
@@ -90,8 +90,6 @@ public:
     explicit UrlBar(QWidget *parent = 0);
     ~UrlBar();
 
-    void activateSuggestions(bool);
-
 public Q_SLOTS:
     void setQUrl(const QUrl &url);
 
@@ -112,6 +110,7 @@ private Q_SLOTS:
     void detectTypedString(const QString &);
     void suggest();
 
+    void manageAdBlock(QPoint);
     void manageFavorites(QPoint);
 
     void refreshFavicon();
@@ -122,9 +121,11 @@ private Q_SLOTS:
     bool isValidURL(QString url);
 
     /**
-     * Load digited url
+     * Load typed url
      */
-    void loadDigitedUrl();
+    void loadTypedUrl();
+
+    void showRSSInfo(const QPoint &);
 
 protected:
     void paintEvent(QPaintEvent *event);
@@ -135,6 +136,9 @@ protected:
     void contextMenuEvent(QContextMenuEvent *event);
     void resizeEvent(QResizeEvent *);
 
+Q_SIGNALS:
+    void focusIn();
+
 private:
     /**
      * Updates right icon position, given its number in the right icons list
